//
// C++ Implementation: BinaryNetwork
//
// Description:
//
//
// Author: Marek Grochowski <grochu@fizyka.umk.pl>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "BinaryNetwork.h"
//---------------------- BINARY NETWORK ------------------------------

BinaryNetwork::BinaryNetwork( )
{
//    classToLearn = (Label)0;
	output_number=0;
	nr_of_outputs=1;
	treshold =0.5;
//    model_name="Neural network with bicentral functions";
}

BinaryNetwork::BinaryNetwork ( Data *data,char *n ) :Network ( data,n )
{
	output_number=0;
	nr_of_outputs=1;
	treshold = 0.5;
//    if(n) model_name=n;
//    else model_name="Neural network with bicentral functions";
}

void BinaryNetwork::DisplayMagicNeuronsOutput()
{
	unsigned long nr_of_signals=data->GetNumberOfVectors();
	float *input=NULL,output=0;

	cout << "Output\t->\tActivation\n";
	for ( unsigned long i=0;i<nr_of_signals;i++ )
	{

		input=data->GetInput ( i );
		output=data->GetOutput ( i,output_number );
		CalculateActivation ( input );

		cout << output << " -> ";
		//  this->First->Next->DisplayNeuronsOutput();
		Last->DisplayNeuronsOutput();
		cout << endl;
	}
}

Network* BinaryNetwork::DuplicateNetwork()
{
	BinaryNetwork *net= new BinaryNetwork ( data );
	if ( !net ) return NULL;
	this->CopyNetworkArchitectureTo ( net );
	this->CopyNetworkParametersTo ( net );
	net->output_number=output_number;
	return net;
}


float BinaryNetwork::CalculateNetworkError( )
{
	unsigned long nr_vectors=data->GetNumberOfVectors();
	float *input=NULL;
	float *output=NULL;

	t_info.Clear();
	o_info[0].Clear();
	t_info.wektorow=data->GetNumberOfVectors();

	for ( unsigned long i=0;i<nr_vectors;i++ )
	{
		input=data->GetInput ( i );
		output=data->GetOutput ( i );
		CalculateActivation ( input );

		float out_j = output[output_number];
		float activ_j   = Last->GetActivation ( 0 );
		float tmp       = out_j-activ_j;

		float mseError = tmp*tmp;

		t_info.ms_error += mseError* ( 1-param.lambda_factor*out_j )
		                   - param.reward_factor*out_j*activ_j
		                   + param.penalty_factor* ( 1-out_j ) *activ_j;
		t_info.mseError += mseError;


		//Jezeli neuron sie wzbudza a nie powinien to uznajemy
		//to za blad
		if ( activ_j>treshold && out_j==0 )
			o_info[0].blad0++;

		//jezeli sie nie wzbudza a powienien to za niedouczenie
		if ( activ_j<=treshold && out_j==1 )
			o_info[0].blad1++;

		//ile poprawnie jest klasyfikowanych
		if ( activ_j>treshold && out_j==1 )
			o_info[0].poprawne1++;

		if ( activ_j <=treshold && out_j==0 )
			o_info[0].poprawne0++;
	}

	//      t_info.accuracy=(float)(t_info.poprawne1+t_info.poprawne0)/(float)t_info.wektorow;
	return ( t_info.ms_error );
}

void BinaryNetwork::AddHiddenNode()
{
	Node *n = this->CreateNode ( param.hidden );
	First->Next->Add ( n,true );

	n->InitNode();

	Last->SetAllWeightsInLayer ( 1 );
	Last->FreezeAllNodesInLayer();
}

/* uczenie sieci - zaczynamy od pojedynczego neuronu ukrytego (magicznego)
                    uczymy N sieci i wybieramy ta o najmniejszym bledzie
                    dokladamy neuron i powtarzamy dopuki ktorys z
                    przypadkow dodatnich nie jest zaklasyfikowany
                    poprzednio nauczone neurony zamrazamy i ustawiamy bias na
                    nachylenie na duza wartosc tak aby na wyjsciach miec ostre granice i
                    neurony ukryte aktywowaly sie watosciami 1 i 0 */
void BinaryNetwork::MyHeuristic ( int nr_of_networks )
{
	network_parameters rem_param=param;
	int attemptsCount = 0;
	Data *oryginaldata = data;
	Data *newdata = NULL;
	BitVector sel ( data->GetNumberOfVectors() );
	long nr_learned = 0;
	int total_attempts = 0, total_initializations = 0, fail_attempts = 0;
	LearnAndFreez();
	total_attempts ++;
	total_initializations+=param.number_of_initiatializations;


	if ( param.vector_selection )
	{
		nr_learned = LearnedSelection ( sel );

		if ( debug_level > 0 )
		{
			cout << "\n nauczylem sie " << nr_learned << " wektorow \n Do uczenia pozostalo\n";
			for ( long i=0;i<data->GetNumberOfVectors();i++ )
				if ( sel.GetBit ( i ) ) cout << i << " ";
			cout << endl;
		}

		newdata = oryginaldata->SelectVectors ( sel );
		data=newdata;
	}

	BinaryNetwork *bestNetwork = ( BinaryNetwork * ) this->DuplicateNetwork();
	int currentWrong=GetNumberOfWrongClasified ( 0 );
	cout << "           good neuron - " << currentWrong << " errors "; PrintError(); cout <<  endl;
	float best_error=t_info.ms_error;
	int lastWrong = currentWrong;

	while ( currentWrong )
	{
		param=rem_param;
		AddHiddenNode();
		LearnAndFreez();
		total_attempts++;
		total_initializations+=param.number_of_initiatializations;

		if ( param.vector_selection )
		{
			data = oryginaldata;
			nr_learned = LearnedSelection ( sel );

			if ( debug_level>0 )
			{
				cout << "\n nauczylem sie " << nr_learned << " wektorow \n Do uczenia pozostalo\n";
				for ( long i=0;i<data->GetNumberOfVectors();i++ )
					if ( sel.GetBit ( i ) ) cout << i << " ";
				cout << endl;
			}
			if ( newdata ) delete newdata;
			newdata = oryginaldata->SelectVectors ( sel );
			data=newdata;
		}

		float error = t_info.ms_error;
		currentWrong=GetNumberOfWrongClasified ( 0 );

		if ( error<best_error && currentWrong < lastWrong )
		{
			cout << " attempt " << attemptsCount+1 << " good neuron - " << currentWrong << " errors "; PrintError(); cout << endl;

			attemptsCount=0;
			best_error=error;
			bestNetwork->CopyNetwork ( *this );
			lastWrong = currentWrong;
		}
		else
		{
			cout << " attempt " << attemptsCount+1 << " BAD neuron! - deleting \n";
			attemptsCount++;
			( ( Network * ) this )->CopyNetwork ( *bestNetwork );
			fail_attempts++;
			if ( param.number_of_attempts > 0 && attemptsCount > param.number_of_attempts-1 ) break; // do trzech razy sztuka
			//  if(s > 0) break;
		}

	}
	cout << "Total: attempts " << total_attempts << " fail_attempts " << fail_attempts << " initializations " << total_initializations << endl;
	data=oryginaldata;
	delete bestNetwork;
	if ( newdata ) delete newdata;
}




void BinaryNetwork::DisplayNeuronsProjection ( Layer *l,int type )
{
	int nr_of_vec=data->GetNumberOfVectors();
	int nr_of_neurons=l->GetNrOfNeurons();

	struct projection **neuronsTab = new struct projection *[nr_of_neurons];
	struct projection *activtab=NULL;

	for ( int k=0;k<nr_of_neurons;k++ )
		neuronsTab[k]=new struct projection[nr_of_vec+2];

	for ( int i=0;i<nr_of_vec;i++ )
	{
		CalculateActivation ( data->GetInput ( i ) );

		Node *position=l->first_neuron;
		for ( int k=0 ;position!=NULL;k++,position=position->Next )
		{
			activtab=neuronsTab[k];
			activtab[i].input_nr=i;
			activtab[i].activation=position->GetProjection();
			activtab[i].output=data->GetOutput ( i,output_number );
		}
	}

	Node *position=l->first_neuron;
	for ( int k=0 ;position!=NULL;k++,position=position->Next )
	{
		activtab=neuronsTab[k];
		activtab[nr_of_vec].activation=position->GetLeftBias();
		activtab[nr_of_vec+1].activation=position->GetRightBias();
		activtab[nr_of_vec].input_nr=-1;
		activtab[nr_of_vec+1].input_nr=-2;
	}

	//sortowanie wg. aktywacji
	for ( int k=0;k<nr_of_neurons;k++ )
	{
		activtab=neuronsTab[k];
		for ( int i=0;i<nr_of_vec;i++ )
			qsort ( activtab,nr_of_vec+2,sizeof ( struct projection ),compare );
	}

	if ( type==0 )
	{
		for ( int k=0;k<nr_of_neurons;k++ )
			cout << "neuron=" <<  k+1 << "\t";

		cout << endl;

		for ( int i=0;i<nr_of_vec+2;i++ )
		{
			for ( int k=0;k<nr_of_neurons;k++ )
			{
				activtab=neuronsTab[k];
				if ( activtab[i].input_nr>=0 )
				{
					cout.width ( 3 );
					cout << std::left << ( int ) activtab[i].output << " (";
					cout.width ( 5 );
					cout.precision ( 2 );
					cout << std::right << activtab[i].activation << ")\t";
				}
				else if ( activtab[i].input_nr==-1 )
					cout << "--/ ("  << activtab[i].activation << ")\t";
				else if ( activtab[i].input_nr==-2 )
					cout << "--\\ ("  << activtab[i].activation << ")\t";
			}
			cout << endl;
		}
	}

	if ( type==1 )
	{
		Node *position=l->first_neuron;
		for ( int k=0;k<nr_of_neurons;k++,position=position->Next )
		{
			cout << "(" << k+1 << ") ";
			activtab=neuronsTab[k];
			for ( int i=0;i<nr_of_vec+2;i++ )
			{

				if ( activtab[i].input_nr>=0 )
					cout << ( int ) activtab[i].output;
				else if ( activtab[i].input_nr==-1 )
					cout << "/" ;
				else if ( activtab[i].input_nr==-2 )
					cout << "\\" ;

			}
			cout << "\t";
		}
	}

	for ( int k=0;k<nr_of_neurons;k++ ) delete [] neuronsTab[k];
	delete [] neuronsTab;
}

void BinaryNetwork::DisplayMagicNeuronsProjection ( int type )
{
	DisplayNeuronsProjection ( Last->Previus,type );
}

void BinaryNetwork::Run ( Data * data )
{
	this->data=data;

	nr_of_inputs = data->GetNumberOfFeatures();
	nr_of_outputs = 1;

	if ( o_info ) delete [] o_info;
	o_info = new struct output_info[nr_of_outputs];
	if ( !o_info ) throw new MemoryException ( "o_info (BinaryNetwork::Run)" );

	int hiddenLayersCount = ( init_param.initial_nr_of_hidden_neurons == 0 ) ? 0 : 1 ;
	CreateNetwork ( nr_of_inputs,nr_of_outputs,hiddenLayersCount );

	if ( param.freezOutput )
	{
		Node *n = Last->first_neuron;
		n->SetWeights ( 1.0 );
		n->SetBias ( ( param.output == sigmoid ) ? 0.5 : 0.0 );
		n->SetSlope ( ( param.output == sigmoid ) ? 2.0 : 1.0 );
		n->FreezeNode();
	}
	//Last->SetAllWeightsInLayer(1);
	//Last->FreezeAllNodesInLayer();

	SetSlope ( param.slope );
//    EstimateBias( data );
	if ( param.network_architecture == static_network ) BestOfNInitiatins ( param.number_of_initiatializations );
	else if ( param.hammingClustering ) HeuristicWithHammingClustering();
	else if ( param.initiation == _rand ) MyHeuristic ( param.number_of_initiatializations );
	else if ( param.initiation == _copy ) HeyristicWithWeightCopy();
}

void BinaryNetwork::CalculateBackSignal ( float * outputs )
{
	float l_factor = param.lambda_factor;
	float reward_factor = param.reward_factor;
	float penalty_factor = param.penalty_factor;

	float activation = Last->GetActivation ( 0 );
	float output = outputs[output_number];
	float backError = ( activation-output ) * ( 1.0-l_factor )
	                  - reward_factor * output * activation
	                  + penalty_factor * ( 1.0-output ) * activation;

	( ( Output* ) Last )->SetBackSignal ( &backError );

	Layer *l=Last->Previus;
	while ( l )
	{
		l->CalculateBackSignal();
		l=l->Previus;
	}
}

int BinaryNetwork::LearnAndFreez( )
{

	BestOfNInitiatins ( param.number_of_initiatializations );
	SetSlope ( 200 );
	First->Next->FreezeAllNodesInLayer();
	float error=CalculateNetworkError();
	int nr_wrong=GetNumberOfWrongClasified ( 0 );

	if ( debug_level>0 )
	{
		cout << "\n\noutput nr = " << output_number << "  wrong = " << nr_wrong << endl;
		cout << " Best of " << param.number_of_initiatializations << " networks \n";
		//  DisplayWeights();
		PrintError();
		cout << endl;
//       DisplayMagicNeuronsOutput();
		DisplayMagicNeuronsProjection ( 1 );
		cout << *this;
	}
	return nr_wrong;
}

int* BinaryNetwork::GetKseparability ( int view )
{
	Layer *l=Last->Previus;

	int nr_of_vec=data->GetNumberOfVectors();
	int nr_of_neurons=l->GetNrOfNeurons();

	struct projection **neuronsTab = new struct projection *[nr_of_neurons];
	struct projection *activtab=NULL;

	for ( int k=0;k<nr_of_neurons;k++ )
		neuronsTab[k]=new struct projection[nr_of_vec+2];

	for ( int i=0;i<nr_of_vec;i++ )
	{
		CalculateActivation ( data->GetInput ( i ) );

		float *projections = l->GetProjections();
		float output = data->GetOutput ( i,output_number );

		for ( int k=0 ;k<nr_of_neurons;k++ )
		{
			activtab=neuronsTab[k];
			activtab[i].input_nr=i;
			activtab[i].activation=projections[k];
			activtab[i].output=output;
		}
		delete [] projections;
	}

	Node *position=l->first_neuron;
	for ( int k=0 ;position!=NULL;k++,position=position->Next )
	{
		activtab=neuronsTab[k];
		activtab[nr_of_vec].activation=position->GetLeftBias();
		activtab[nr_of_vec+1].activation=position->GetRightBias();
		activtab[nr_of_vec].input_nr=-1;
		activtab[nr_of_vec+1].input_nr=-2;
	}

	//sortowanie wg. aktywacji
	for ( int k=0;k<nr_of_neurons;k++ )
	{
		activtab=neuronsTab[k];
		for ( int i=0;i<nr_of_vec;i++ )
			qsort ( activtab,nr_of_vec+2,sizeof ( struct projection ),compare );
	}

	int *kSep = new int[nr_of_neurons];
	if ( !kSep ) throw new MemoryException ( "Tablica k-sep nie powstala" );

	for ( int k=0;k<nr_of_neurons;k++ )
	{
		kSep[k]=1;
		activtab=neuronsTab[k];
		float lastOutput = activtab[0].output;
		for ( int i=1;i<nr_of_vec+2;i++ )
		{
			float currentOutput = activtab[i].output;
			if ( activtab[i].input_nr >= 0 && currentOutput != lastOutput )
			{
				kSep[k]++;
				lastOutput = currentOutput;
			}
		}
	}

	if ( debug_level > 2 || view == 2 )
	{
		for ( int k=0;k<nr_of_neurons;k++ )
			cout << "neuron=" <<  k+1 << "\t";

		cout << endl;

		for ( int i=0;i<nr_of_vec+2;i++ )
		{
			for ( int k=0;k<nr_of_neurons;k++ )
			{
				activtab=neuronsTab[k];
				if ( activtab[i].input_nr>=0 )
				{
					cout.width ( 3 );
					cout << std::left << ( int ) activtab[i].output << " (";
					cout.width ( 5 );
					cout.precision ( 2 );
					cout << std::right << activtab[i].activation << ")\t";
				}
				else if ( activtab[i].input_nr==-1 )
					cout << "--/ ("  << activtab[i].activation << ")\t";
				else if ( activtab[i].input_nr==-2 )
					cout << "--\\ ("  << activtab[i].activation << ")\t";
			}
			cout << endl;
		}
	}

	if ( debug_level > 0 || view == 1 )
	{
		Node *position=l->first_neuron;
		for ( int k=0;k<nr_of_neurons;k++,position=position->Next )
		{
			cout << "\n(" << k+1 << ") ";
			activtab=neuronsTab[k];
			for ( int i=0;i<nr_of_vec+2;i++ )
			{

				if ( activtab[i].input_nr>=0 )
					cout << ( int ) activtab[i].output;
				else if ( activtab[i].input_nr==-1 )
					cout << "/" ;
				else if ( activtab[i].input_nr==-2 )
					cout << "\\" ;

			}
			cout << " k= " << kSep[k] << endl;
		}
	}

	for ( int k=0;k<nr_of_neurons;k++ ) delete [] neuronsTab[k];
	delete [] neuronsTab;

	return kSep;
}

void BinaryNetwork::HeyristicWithWeightCopy( )
{
	network_parameters rem_param=param;
	int s = 0;

	LearnAndFreez(); // pierwszy neuron uczony tradycyjnie

	BinaryNetwork *bestNetwork = ( BinaryNetwork * ) this->DuplicateNetwork();
	int currentWrong=GetNumberOfWrongClasified ( 0 );
	cout << " good neuron - " << currentWrong << " errors "; PrintError(); cout << endl;
	float best_error=t_info.ms_error;
	int lastWrong = currentWrong;

	while ( currentWrong )
	{
		param=rem_param;

		//dodajemy nowy neuron z tymi samymi wagami co ostatnio nauczony
		Node *n = this->CreateNode ( param.hidden );
		Node *prev = First->Next->FindLastNode();

		First->Next->Add ( n,true );
		n->CopyWeightsFromNode ( prev );
		n->SetSlope ( init_param.slope );
		//zamrazamy wyjscie
		Last->SetAllWeightsInLayer ( 1 );
		Last->FreezeAllNodesInLayer();

//        EstimateBias( data );

		Learning ( param.strategy_of_learning );

		float error = t_info.ms_error;
		currentWrong=GetNumberOfWrongClasified ( 0 );

		if ( error<best_error && currentWrong < lastWrong )
		{
			cout << " good neuron - " << currentWrong << " errors "; PrintError(); cout << endl;

			s=0;
			best_error=error;
			bestNetwork->CopyNetwork ( *this );
			lastWrong = currentWrong;
		}
		else
		{
			cout << " BAD neuron! - deleting \n";
			s++;
			( ( Network * ) this )->CopyNetwork ( *bestNetwork );
			if ( s > 2 ) break; // do trzech razy sztuka
			//  if(s > 0) break;
		}

	}
	delete bestNetwork;
}

BitVector * BinaryNetwork::LearnedSelection( )
{
	unsigned long nr_vectors=data->GetNumberOfVectors();
	float *input=NULL;
	float *output=NULL;

	BitVector *selection = new BitVector ( nr_vectors );
	selection->Clear ( 1 );

	for ( unsigned long i=0;i<nr_vectors;i++ )
	{
		input=data->GetInput ( i );
		output=data->GetOutput ( i );
		CalculateActivation ( input );

		float out_j = output[output_number];
		float activ_j   = Last->GetActivation ( 0 );


		//ile poprawnie jest klasyfikowanych
		if ( activ_j>treshold && out_j==1 )
			selection->SetBit ( i, 0 )
			;
	}

	return selection;
}

long BinaryNetwork::LearnedSelection ( BitVector & sel )
{
	unsigned long nr_vectors=data->GetNumberOfVectors();
	if ( nr_vectors != sel.Size() )
	{
		throw new Exception ( "size of data and bitvector don't match in LearnedSelection()" );
	}

	float *input=NULL;
	float *output=NULL;

	sel.Clear ( 1 );
	long nr_learned = 0;

	for ( unsigned long i=0;i<nr_vectors;i++ )
	{
		input=data->GetInput ( i );
		output=data->GetOutput ( i );
		CalculateActivation ( input );

		float out_j = output[output_number];
		float activ_j   = Last->GetActivation ( 0 );


		//ile poprawnie jest klasyfikowanych
		if ( activ_j>treshold && out_j==1 )
		{
			sel.SetBit ( i, 0 );
			nr_learned ++;
		}
	}
	return nr_learned;
}

void BinaryNetwork::HeuristicWithHammingClustering()
{
	network_parameters newParameters = param;
	float *weights = new float[this->nr_of_inputs];
	this->First->Next->DelAllNodes();
	//  this->Last->first_neuron->SetSlope(200);

	int total_attempts = 0, total_initializations = 0, fail_attempts = 0;
	int attemptsCount = 0;

	Data *oryginaldata = data;
	Data *newdata = new Data ( *data );
	newdata->RefreshDataInfo();

	int vectors = data->GetNumberOfVectors();
	int currentWrong = vectors;

	BinaryNetwork *finalNetwork = new BinaryNetwork();
	BinaryNetwork *bestNetwork = new BinaryNetwork();

	finalNetwork->Configure ( ( Parameters & ) param );
	finalNetwork->ConnectWithData ( oryginaldata );
	finalNetwork->SetLabelToLearn ( output_number );

	int hid=0;
	finalNetwork->CreateNetwork ( nr_of_inputs,nr_of_outputs,1,&hid );
	Node *n = finalNetwork->Last->first_neuron;
	n->SetBias ( ( param.output == sigmoid ) ? 0.5 : 0.0 );
	n->SetSlope ( 200 );
	BitVector sel ( vectors );

	float best_error= ( float ) currentWrong+1000;
	int lastWrong = currentWrong;
	int toLearn = data->labelsInfo->GetNumberOfVectorsInClass ( output_number );

	while ( currentWrong && newdata->labelsInfo->GetNumberOfVectorsInClass ( output_number ) > 0 )
	{

		HammingClustering hc;
		long nr_learned = 0;

//        cout << "New data, przed HC\n";     newdata->DisplayData();

		newdata->RefreshDataInfo();
		Data *currentData = hc.Learn ( newdata );
		int currentFearures = hc.GetNumbetOfSelectedFeatures();
		int *currentFeaturesId = hc.GetSelectedFeatures();
		cout << endl;
		cout << hc;
//         currentData->DisplayData();
//            cout << *currentData->labelsInfo << endl;
		cout << endl ;
		cout << "Hamming clustering data output: " << endl;
		currentData->DisplayDataInfo();
		cout << endl;

		BinaryNetwork *currentNetwork = new BinaryNetwork();
		newParameters.initial_nr_of_hidden_neurons = 1;
		newParameters.network_architecture = static_network;
		newParameters.hammingClustering = false;

		currentNetwork->SetLabelToLearn ( output_number );
		currentNetwork->Configure ( ( Parameters & ) newParameters );
		currentNetwork->SetDebugLevel ( 0 );
		currentNetwork->Run ( currentData );

		total_attempts++;
		total_initializations+=param.number_of_initiatializations;

		delete currentData;
		currentData = NULL;
		/*  cout << "podsiec\n";
		  cout << *currentNetwork << endl;
		  currentNetwork->PrintError();
		  cout << endl;
		*/
		Node *neuron = currentNetwork->First->Next->first_neuron;
		float *wsel = neuron->GetWeights();
		Node *newNode = neuron->Clone();
		newNode->SetSlope ( 200 );
		for ( int i=0;i<nr_of_inputs;i++ ) weights[i]=0.0;
		for ( int i=0;i<currentFearures;i++ ) weights[currentFeaturesId[i]]=wsel[i];
		delete [] wsel;


		finalNetwork->AddHiddenNode ( newNode,weights );
		finalNetwork->Last->SetAllWeightsInLayer ( 1.0 );
		finalNetwork->ConnectWithData ( oryginaldata );
		oryginaldata->RefreshDataInfo();
		finalNetwork->CalculateNetworkError();
//        finalNetwork->DisplayMagicNeuronsOutput();
//        cout << "intermedio network:\n";
//        finalNetwork->DisplayMagicNeuronsOutput();
		/*        finalNetwork->DisplayMagicNeuronsProjection(1);
		        cout << *finalNetwork;
		        finalNetwork->PrintError();
		        cout << endl;*/


		float error = finalNetwork->t_info.ms_error;
		currentWrong=finalNetwork->GetNumberOfWrongClasified ( 0 );
		nr_learned = finalNetwork->LearnedSelection ( sel );
//       finalNetwork->DisplayMagicNeuronsProjection(1);


		if ( error<best_error && currentWrong < lastWrong )
		{
			cout << " attempt " << attemptsCount+1 << " good neuron - " << currentWrong << " errors "; finalNetwork->PrintError(); cout << endl;

			best_error=error;
			attemptsCount=0;
			bestNetwork->CopyNetwork ( *finalNetwork );
			lastWrong = currentWrong;
			toLearn-=nr_learned;
			delete newdata;
			newdata = oryginaldata->SelectVectors ( sel );

		}
		else
		{
			cout << " attempt " << attemptsCount+1 << " BAD neuron! - deleting \n";
			attemptsCount++;
			finalNetwork->CopyNetwork ( *bestNetwork );
			fail_attempts++;
			if ( param.number_of_attempts > 0 && attemptsCount > param.number_of_attempts-1 ) break;
		}

		delete currentNetwork;
	}
	this->CopyNetwork ( *bestNetwork );

	cout << "ostatecznie\n" << *this << endl;
	delete [] weights;
	delete finalNetwork,bestNetwork;


	cout << "Total: attempts " << total_attempts << " fail_attempts " << fail_attempts << " initializations " << total_initializations << endl;
	data=oryginaldata;

	return ;
}

void BinaryNetwork::AddHiddenNode ( Node * n, float * weights )
{
	First->Next->Add ( n,weights );
}
