//
// C++ Interface: BinaryNetwork
//
// Description: 
//
//
// Author: Marek Grochowski <grochu@fizyka.umk.pl>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _BINARYNETWORK_H
#define _BINARYNETWORK_H

#include "Network.h"
#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <vector>
#include "Model.h"
#include "Layer.h"
#include "Data.h"
#include "exceptions.h"
#include "Parameters.h"
#include "BitVector.h"
#include "HammingClustering.h"

// szkielet sieci z jednym wyjsciem, uzywany w BinaryMLP lub w KNetwork, dla problemow dwuklasowych lub wieloklasowych
class BinaryNetwork : public Network
{
    void AddHiddenNode();
    void AddHiddenNode(Node *n,float *weights);
  //  Label classToLearn;

    void CalculateBackSignal(float *output);
    int LearnAndFreez();
 protected:
    int output_number;                     ///< okresla ktora z klas (etykiet) ma byc uczana
    float treshold;
    BitVector *LearnedSelection();
    long LearnedSelection(BitVector &sel);
public:
    BinaryNetwork(Data *data,char *name=NULL);
    BinaryNetwork();
//    void backSignal(float *output,float l_factor=0.0,float reward_factor=0.0,float penalty_factor=0.0);
    void DisplayMagicNeuronsOutput();
    void DisplayMagicNeuronsProjection(int type=0);
    int* GetKseparability(int view=0);
    void DisplayNeuronsProjection(Layer *l,int type=0);
    float CalculateNetworkError();
    void SetTreshold(float value){ treshold = value;}
    float GetTreshold(){ return treshold;}
    void SetLabelToLearn(int i)
    {
        output_number = i;
    }
    
    void Run(Data *data);
    void MyHeuristic(int nr_of_networks=-1);
    void HeuristicWithHammingClustering();
    void HeyristicWithWeightCopy();
    Network* DuplicateNetwork();

    BinaryNetwork& operator=(BinaryNetwork &bn)
    {
        CopyNetworkWeightsFrom(&bn);
        output_number=bn.output_number;
        bn.CopyNetworkParametersTo(this);
        return *this;
    }
};




#endif

