
#ifndef __DATA_H
#define __DATA_H
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <set>
#include "BitVector.h"
#include "exceptions.h"
#include "math.h"
#include "LabelsReprezentation.h"
#include "Parameters.h"

using namespace std;

typedef float Label;


struct DataStatistics
{
     float min;
     float max;
     float mean;
     float std;
     float var;                               
};


class Data
{
protected:
    
    int is_a_copy;
//    int number_of_classes;
    unsigned long number_of_features;
    unsigned long number_of_signal_vectors;
    int number_of_outputs;
    
    struct InputData
    {
        float *InputS;
        float *OutputS;
        Label classLabel;
    }
    **signal;

    typedef  struct InputData* Vector;
    struct DataStatistics *dstat;

    int CheckDataFile(ifstream &file,data_format t);
    void ClearDataSet();
    void ComputeNumberOfVectorsInClasses();
//    void CreateLabelsTable();
    
public:
    
    Data();
    Data(const Data &d);
    ~Data();
    
//    long *vectors_in_class;
//    Label *class_labels;
    char *data_name;
    LabelsReprezentation *labelsInfo;
    
    void DisplayStatistics();
    void RefreshStatistics();
    void RefreshDataInfo();
    void CreateEmptyDataTable(long nr_vec, long nr_feat);
    void CreateDiagonalReprezentationOfLabels();
    void Add(Data *d);
    
    static Data *CreateEmptyDataset(long nr_vec,long nr_feat);
            
    DataStatistics &GetStatisticsOfFeature(long i)
    {
        return dstat[i];
    }
    
    int GetNumberOfFeatures()
    {
        return number_of_features;
    }

    int GetNumberOfClasses()
    {
        return labelsInfo->GetNumberOfLabels();
    }

    int GetNumberOfVectors()
    {
        return number_of_signal_vectors;
    }
    
    int GetNumberOfOutputs()
    {
        return number_of_outputs;
    }

    int GetNrVectorsInClass(int c);

    float *GetInput(long i)
    {
        return signal[i]->InputS;
    }

    float *GetOutput(long i)
    {
        return signal[i]->OutputS;
    }

    float GetOutput(long i,int o)
    {
        return signal[i]->OutputS[o];
    }

    Label GetLabel(long i)
    {
        return signal[i]->classLabel;
    }
    
    void SetLabel(long vec_id,Label lab)
    {
        signal[vec_id]->classLabel=lab;
  //      signal[vec_id]->OutputS=labelsInfo->Label2Output(lab);
    }

    void ReadData(char *file_name);
    void ReadData(char *file_name,data_format f);
    void ReadData(ifstream &in,data_format f);
    
    void Save(char *filename);
    void Save(char *file_name,data_format f);
    
    void SaveData(ofstream &o);
    void SaveData(ofstream &o,data_format f);
//    void ShuffleData();
    void DisplayData();
    void DisplayDataInfo();
    void DisplayLabelsInfo();
    Data *SelectVectors(BitVector &bitvector);
    Data *SelectFeatures(BitVector &bitvector);
    Data *SelectFeaturesAndVectors(BitVector & vectors,BitVector &features);
    Data *Clone();

    friend ostream& operator<<(ostream &o,Data &d)
    {
        d.RefreshDataInfo();
        o << " -- Dataset ------------- \n";
        o << "No. of vectors " <<  d.number_of_signal_vectors << endl;
        o << "No. number_of_features   " << d.number_of_features << endl;
        o << "Number of outputs " << d.number_of_outputs << endl;
        return o;
    }
};

#endif
