//
// C++ Implementation: HammingClustering
//
// Description: 
//
//
// Author: Marek Grochowski <grochu@fizyka.umk.pl>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "HammingClustering.h"

Data * HammingClustering::Learn(Data * d)
{
    int ninp = d->GetNumberOfFeatures();
    int vectorsCount = d->GetNumberOfVectors();
    
    int nplus = d->GetNrVectorsInClass(classToLearn);
    int nminus = vectorsCount - nplus;
    
    int *plusInd = new int[nplus];
    int *minusInd = new int[nminus];
    
    int plusCount=0,minusCount=0;
    
    for(int i=0;i<vectorsCount;i++) 
        if(d->GetOutput(i,classToLearn)) plusInd[plusCount++]=i; 
            else minusInd[minusCount++]=i;
  
    int nvset =0;
    int nwapp = 0,niapp=0,njapp=0;
  
    int *japp = new int[ninp];
    iset = new int[ninp];
    int ibest = 0;
    
    int *vset = new int[nplus];
    int *viset = new int[nplus];
    int *wset = new int[nplus];
  
    niapp = ninp;
    for (int i = 0; i < ninp; i++)     iset[i] = i;
    for (int i = 0; i < nplus; i++)   wset[i] = plusInd[i];
    nwapp = nplus;
    int thresh = max(nplus/2,2*ninp);
    if (thresh > nplus)    thresh = nplus;
  
    do {
        nvset = 0;
        njapp = niapp;
        RandomID rand(niapp);
        for (int i = 0; i < niapp; i++)     japp[i] = i;
    
        for (int i = 0; i < niapp; i++) 
        {
           // int ind = njapp*drand48();
            int ind = *rand;
            int itemp = iset[japp[ind]];
            iset[japp[ind]] = iset[niapp-1];
            iset[niapp-1] = itemp;
      
            int nviset = 0;
            for (int j = 0; j < nwapp; j++) 
            {
                int k = 0;
                for (; k < nminus && !EqualMask (d->GetInput(wset[j]), d->GetInput(minusInd[k]), iset, niapp-1); k++)
                    ;
                if (k == nminus)  viset[nviset++]=wset[j];
            }
      
            if (nviset > nvset) 
            {
                ibest = japp[ind];
                nvset = nviset;
                
                for (int j = 0; j < nviset; j++)  vset[j] = viset[j];
            }
        
            itemp = iset[japp[ind]];
            iset[japp[ind]] = iset[niapp-1];
            iset[niapp-1] = itemp;
      
            itemp = japp[njapp-1];
            japp[njapp-1] = japp[ind];
            japp[ind] = itemp;
            njapp--;
    
            if (nvset == nplus) // czy tu odjac jeden
                break;
        }
        
        if (nvset >= thresh) 
        {
            niapp--;
            for (int i = ibest; i < niapp; i++)   iset[i] = iset[i+1];
            nwapp = nvset;
            for(int j=0; j< nwapp ; j++) wset[j] = vset[j];
/*            
            cout << "niapp = " << niapp << "  ibest = " << ibest << " nwapp = " << nwapp << endl;
            cout << "vectors: ";
            for(int j=0; j< nwapp ; j++) cout << wset[j] << " ";
            cout << endl ;
            cout << "features: ";
            for(int j=0; j< niapp ; j++) cout << iset[j] << " ";
            cout << endl << endl;*/
            
        }
    } while (nvset >= thresh && niapp > 0);

    
    niset = niapp;
    int nwset = nwapp;
    
    int k=0;
    int *wplus = new int[nplus];
    for (int i = 0; i < nwset; i++) 
    {
      wplus[k] = wset[i];
      int j=0;
      for (; j < k && !EqualMask (d->GetInput(wplus[j]),d->GetInput(wplus[k]),iset,niset); j++) 
            ;
        if (j == k)
            k++;
    }
    int nwplus = k;
    
    k=0;
    int *wminus = new int[nminus];
    for (int i = 0; i < nminus; i++) 
    {
        wminus[k] = minusInd[i];
        int j=0;
        for (; j < k && !EqualMask (d->GetInput(wminus[j]),d->GetInput(wminus[k]),iset,niset); j++)
            ;
        if (j == k)
            k++;
    }
    int nwminus = k;
    
    BitVector vectorSelection(vectorsCount);
    vectorSelection.Clear(0);
    
    featureSelection = new BitVector(ninp);
    featureSelection->Clear(0);

    for(int i=0;i<niset;i++) featureSelection->SetBitOn(iset[i]);
    
    for(int i=0;i<nwplus;i++) vectorSelection.SetBitOn(wplus[i]);
    for(int i=0;i<nwminus;i++) vectorSelection.SetBitOn(wminus[i]);
    
    delete [] japp;
    delete [] vset;
    delete [] viset;
    delete [] wset;
    delete [] wminus;
    delete [] wplus;
    
    return d->SelectFeaturesAndVectors(vectorSelection,*featureSelection);
}

Data * HammingClustering::Run(Data * d)
{
    return d->SelectFeatures(*featureSelection);
}


HammingClustering::~ HammingClustering()
{
    if (featureSelection) delete featureSelection;
    if (iset) delete [] iset;
    iset = NULL;
    featureSelection = NULL;
}

int HammingClustering::EqualMask(float * patt1, float * patt2, int *mask, int nmask)
{
 int  i;

  for (i = 0; i < nmask; i++) {
    int k = mask[i];
    if (patt1[k] != patt2[k])  break;
  }
  return (i == nmask);
}

int HammingClustering::EqualPattern(float * patt1, float * patt2, int ninp)
{
  int i=0;

  for (; i < ninp && patt1[i] == patt2[i]; i++)
    ;
  return (i == ninp);
}
