#include "KNetwork.h"

KNetwork::KNetwork()
{
	init_param=NULL;
	data=NULL;
	model_name=NULL;
	netTable=NULL;
	nrOfNetworks=0;
	init_param=NULL;
}

// KNetwork::KNetwork(Data *data)
// {
// 	init_param=NULL;
// 	CreateKNetwork(data);
// }

void KNetwork::CreateKNetwork(Data *d)
{
    model_name="Klasyfikator zlozony (K-Network)";
    data=d;
    nrOfNetworks=data->GetNumberOfClasses();
//    if(nrOfNetworks == 2 ) nrOfNetworks=1;  //wystarczy tylko jedna siec
   
    netTable = new BinaryNetwork[nrOfNetworks];
    if(!netTable) throw new MemoryException("nie powstal komitet");

    for(unsigned int i=0;i<nrOfNetworks;i++)
    {
        netTable[i].SetLabelToLearn(i);
        if(init_param) netTable[i].Configure(*init_param);
    }
}

//Konfiguracja poczatkowa wszystkich sieci
void KNetwork::Configure(Parameters &p)
{
	init_param=&p;
	for(unsigned int i=0;i<nrOfNetworks;i++)
        netTable[i].Configure(p);

}

//Uczenie modelu
void KNetwork::Learn(Data *d)
{
    data=d;
    CreateKNetwork(d);
    for(unsigned int i=0;i<nrOfNetworks;i++)
        netTable[i].Run(d);
}


//test pojedynczego wektora, rezultatem jest etykieta klasy
float KNetwork::Test(float *vector)
{
    float *outputs = new float[nrOfNetworks];
    if(!outputs)	throw new MemoryException("komitet nie powstal (Test)");

	for(unsigned int i=0;i<nrOfNetworks;i++)
    {
        netTable[i].CalculateActivation(vector);
        outputs[i]=netTable[i].GetActivation(0);
    }

    //znajduje siec o najwiekszej aktywacji na wyjsciu
    // i przydzielam wektor do tej klasy
    float max=0;
    int index=0;
    for(int i=0;i<nrOfNetworks;i++)
    {
        if(outputs[i]>max)
        {
            max=outputs[i];
            index=i;
		}
    }
	delete [] outputs;
	return data->labelsInfo->GetLabel( index );
}

//Zapis sieci na dysku
void KNetwork::Save(char *file)
{
	ofstream file_out;
	file_out.open(file);
	if(!file_out) throw new FileException(file);
	
	file_out << this->nrOfNetworks;
	for(int i=0;i<nrOfNetworks;i++)
		netTable[i].Save(file_out);

	file_out.close();
}

//odczyt modelu
void KNetwork::Load(char *file)
{
	ifstream file_in;
	file_in.open(file);
	if(!file_in) throw new FileException(file);
	
	file_in >> nrOfNetworks;

	if(netTable) delete [] netTable;
	netTable = new BinaryNetwork[nrOfNetworks];
	if(!netTable) throw new MemoryException("problemy przy zapisie komitetu (Load)");

	for(int i=0;i<nrOfNetworks;i++)
		netTable[i].Load(file_in);

	file_in.close();
}

KNetwork::~KNetwork()
{
    delete [] netTable;
}

//Polaczenie modelu z danymi - konieczne gdy wczytujemy model z pliku
// void KNetwork::ConnectWithData(Data *d)
// {
// 	data=d;
// 	if(!netTable) throw new Exception("Nie ma modelu do ktorego mozna podlaczyc dane (CinnectWithData)");
// 	unsigned int nr_of_outputs = data->GetNumberOfClasses();
// 	if(nr_of_outputs!=nrOfNetworks)	throw new Exception("Cos nie tak z liczba sieci (ConnectWithData)");
// 
//     for(int i=0;i<nrOfNetworks;i++)
//         netTable[i].ConnectWithData(d,i);
// }

float KNetwork::TrainError()
{
	unsigned long correct=0;
	unsigned long nr_vec=data->GetNumberOfVectors();
	for(unsigned long i=0;i<nr_vec;i++)
		if((float)Test(data->GetInput(i))==(float)data->GetLabel(i)) 
			correct++;

	return 1.0-(float)correct/(float)nr_vec;
}

void KNetwork::Reset()
{
	if(init_param) Configure(*init_param);
	if(netTable) delete [] netTable;
	netTable=NULL;
	data=NULL;
	nrOfNetworks=0;
}

int KNetwork::NumberOfNeurons()
{
    int non = 0;
    for(unsigned int i=0;i<nrOfNetworks;i++)
        non += netTable[i].NumberOfNeurons();
    
    return non;
}

long KNetwork::GetNumberOfNullConnections(double epsilon)
{
    long noc = 0;
    for(unsigned int i=0;i<nrOfNetworks;i++)
        noc += netTable[i].GetNumberOfNullConnections(epsilon);
    
    return noc;


}


float KNetwork::GetValueOf(char *)
{
	return NumberOfNeurons();
}