//
// C++ Implementation: KSepNetwork
//
// Description: 
//
//
// Author: Marek Grochowski <grochu@fizyka.umk.pl>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "KSepNetwork.h"

KSepNetwork::KSepNetwork( )
{
}


void KSepNetwork::CreateNetwork( int inputs, int outputs, int hidden, int * hidden_neurons )
{
    Layer::counter=0;
    AddLayer(new Input());

    Node::counter=0;
    for(int i=0;i<inputs;i++)
        First->Add(CreateNode(linear),false);

    Layer *l=new Hidden();
    Node::counter=0;
    int nr_n=0;
    l->Add( CreateNode(linear),false);
    AddLayer(l);
        
    l=new Hidden();
    Node::counter=0;
    l->Add(CreateNode(param.hidden),false);
    AddLayer(l);

    //wyjsciowa
    AddLayer(new Output());

    Node::counter=0;
    for(int i=0;i<outputs;i++)
        Last->Add(CreateNode(param.output),false);

    this->CreateConnection();
    this->InitNetwork();
    this->is_created=1;
}

Network * KSepNetwork::DuplicateNetwork( )
{
    KSepNetwork *net= new KSepNetwork();
    if(!net) return NULL;
    this->CopyNetworkArchitectureTo(net);
    this->CopyNetworkParametersTo(net);
    net->output_number=output_number;
    return net;
}

void KSepNetwork::Run( Data * data )
{
    this->data=data;
   
    nr_of_inputs = data->GetNumberOfFeatures();
    nr_of_outputs = 1;
    
    if(o_info) delete [] o_info;
    o_info = new struct output_info[nr_of_outputs];
    if(!o_info) throw new MemoryException("o_info (ConnectWithData)");
      
    int hiddenLayersCount = 2;
    CreateNetwork(nr_of_inputs,nr_of_outputs,hiddenLayersCount);
    
    Node *n = Last->first_neuron;
    
    n->SetWeights( -1.0f );
    //n->SetBias( (param.output == sigmoid) ? -0.5f : 0.0);
    n->SetBias( (param.output == sigmoid) ? 0.0f : 0.0);
    n->SetSlope( (param.output == sigmoid) ? 2.0 : 1.0 );
    n->FreezeNode();
    
    //Last->SetAllWeightsInLayer(1);
    //Last->FreezeAllNodesInLayer();
    
    SetSlope( param.slope );
    EstimateBias( data );
    
    MyHeuristic(param.number_of_initiatializations);    
}


void KSepNetwork::MyHeuristic( int nr_of_networks )
{
    network_parameters rem_param=param;
    int s = 0;
    
    
    BestOfNInitiatins(param.number_of_initiatializations);
    int currentWrong= GetNumberOfWrongClasified( 0 );
 
    BinaryNetwork *lastNetwork = (BinaryNetwork *)this->DuplicateNetwork();
    cout << " good neuron - " << currentWrong << " errors "; PrintError(); cout << endl;
    
    float best_error=t_info.ms_error;
    int lastWrong = currentWrong;
    
    while(currentWrong)
    {
        param=rem_param;
        AddHiddenNode();
        
        BestOfNInitiatins(param.number_of_initiatializations);
        
        float error = t_info.ms_error;
        currentWrong=GetNumberOfWrongClasified(0);
        
        if(error<best_error && currentWrong < lastWrong)
        {
            cout << " good neuron - " << currentWrong << " errors "; PrintError(); cout << endl;
            
            s=0;
            best_error=error;
            lastNetwork->CopyNetwork(*this);
            lastWrong = currentWrong;
        }
        else
        {
            cout << " BAD neuron! - deleting \n";
            s++;
            ((Network *)this)->CopyNetwork(*lastNetwork);
            if(s > 2) break; // do trzech razy sztuka
           //  if(s > 0) break;
        }
        
    }
    delete lastNetwork;
}

int KSepNetwork::LearnAndFreez( )
{
    BestOfNInitiatins(param.number_of_initiatializations);
    SetSlope(200);
    Last->Previus->FreezeAllNodesInLayer();
    float error=CalculateNetworkError();
    int wrong=GetNumberOfWrongClasified(0);

    if(debug_level>0)
    {
        cout << "\n\noutput nr = " << output_number << "  wrong = " << wrong << endl;
        cout << " Best of " << param.number_of_initiatializations << " networks \n";
        //  DisplayWeights();
        PrintError();
        cout << endl;
 //       DisplayMagicNeuronsOutput();
        DisplayMagicNeuronsProjection(1);
        cout << *this;
    }
    return wrong;
}

void KSepNetwork::AddHiddenNode( )
{
    Node *n = CreateNode( param.hidden );
    Last->Previus->Add(n,true);
    n->InitNode();

    Last->SetAllWeightsInLayer(-1);
    Last->FreezeAllNodesInLayer();
}

// int * KSepNetwork::GetKseparability( int view )
// {
//     return ((BinaryNetwork)this)->GetKseparability();
// }

