#include "Layer.h"

extern void Error(int);
int Node::counter=0;

Layer::Layer()
{
    first_neuron=NULL;
    Next=NULL;
    Previus=NULL;
    counter++;
    number=counter;
    nr_of_neurons=0;
    name=NULL;
    node_type=1;
}

Layer::~Layer()
{
    DelAllNodes();
}


void Layer::Add(Node *n,bool connect)
{
    AddNode(n);
    if(connect && Previus)
    {
        Node *position=Previus->first_neuron;
        while(position)
        {
            n->AddNodeToConnectionTable(position,0);
            position=position->Next;
        }
    }
    if(Next)
    {
        Node *position=Next->first_neuron;
        while(position)
        {
            position->AddNodeToConnectionTable(n,0);
            position=position->Next;
        }
    }
}

void Layer::AddNode(Node *tmp)
{
    if(!tmp) throw new Exception("no node (AddNode)");

    //Teraz dostawienie wezla do listy
    if(!first_neuron)
    {
        first_neuron=tmp;
        first_neuron->Next=NULL;
    }
    else
    {
        Node *position=first_neuron;
        while(position->Next)
            position=position->Next;

        position->Next=tmp;
        position->Next->Next=NULL;
    }

    nr_of_neurons++;
    tmp->number=nr_of_neurons;
}

void Layer::DelNode(Node *remove)
{
    Node *position;

    if(first_neuron==remove)
    {
        first_neuron=first_neuron->Next;
        delete remove ;
    }
    else
    {
        position=first_neuron;
        while(position->Next!=remove && position->Next)
            position=position->Next;
        position->Next=position->Next->Next;
        delete remove ;
        remove=NULL;
    }
    nr_of_neurons--;
}

void Layer::DelAllNodes()
{
    Node *position;

    while(first_neuron)
    {
        position=first_neuron;
        first_neuron=first_neuron->Next;
        delete position;
        nr_of_neurons--;
    }
}

void Layer::DelAllConnections()
{
    Node *position=first_neuron;
    while(position)
    {
        position->DelConnectionTable();
        position=position->Next;
    }
}

void Layer::InitLayer()
{
    Node *position=first_neuron;

    while(position)
    {
        position->InitNode();
        position=position->Next;
    }
}

void Layer::CalculateDiff()
{
    Node *position=first_neuron;
    while(position)
    {
        position->DiffActivation();
        position=position->Next;
    }
}

void Layer::ChangeSlope(float inc)
{
    Node *position=first_neuron;
    while(position)
    {
        position->ChangeSlope(inc);
        position=position->Next;
    }
}

void Layer::SetSlope(float s)
{
    Node *position=first_neuron;
    while(position)
    {
        position->SetSlope(s);
        position=position->Next;
    }
}

/* zwraca aktywacje i-tego neuronu */
float Layer::GetActivation(int i)
{
    int count=0;
    Node *n=first_neuron;

    while(count!=i && n)
    {
        count++;
        n=n->Next;
    }
    return n->GetActivation();
}

float * Layer::GetActivations( )
{
    Node *n=first_neuron;
    float *activTab = new float[nr_of_neurons];
    int  i=0;
    while(n)
    {
        activTab[i++] = n->GetActivation();
        n=n->Next;
    }
    return activTab;
}

float * Layer::GetProjections( )
{
    Node *n=first_neuron;
    float *activTab = new float[nr_of_neurons];
    int  i=0;
    while(n)
    {
        activTab[i++] = n->GetProjection();
        n=n->Next;
    }
    return activTab;
}

// void Layer::ChangeWeights(float learning_factor,float regularization)
// {
//     Node *position=first_neuron;
// 
//     while(position)
//     {
//         position->ChangeWeights(learning_factor,regularization);
//         position=position->Next;
//     }
// }

void Layer::ChangeWeights( const network_parameters & p )
{
    Node *position=first_neuron;
    while(position)
    {
        position->ChangeWeights(p);
        position=position->Next;
    }
}


/*void Layer::backSignal(float *output,float l_factor,float reward_factor,float penalty_factor)
{
    Node *position=first_neuron;
    int counter=0;

    while(position)
    {
        if(!Next) //Warstwa wyjsciowa
        {
            position->CalculateBackSignal(output[counter],l_factor,reward_factor,penalty_factor);
            counter++;
        }
        else
            position->CalculateBackSignal();

        position=position->Next;
    }
}
*/

void Layer::CalculateBackSignal( )
{
    Node *position=first_neuron;
    while(position)
    {
        position->CalculateBackSignal();
        position=position->Next;
    }
}

// void Layer::backSignal(float output,float l_factor,float reward_factor,float penalty_factor)
// {
//     Node *position=first_neuron;
//     int counter=0;
// 
//     while(position)
//     {
//         if(!Next) //Warstwa wyjsciowa
//         {
//             position->CalculateBackSignal(output,l_factor,reward_factor,penalty_factor);
//             counter++;
//         }
//         else
//             position->CalculateBackSignal();
// 
//         position=position->Next;
//     }
// }

// zastapione przez przeladowany operator <<
void Layer::DisplayWeights()
{
    cout << *this;
}

void Layer::SaveLayer(ofstream &file)
{
    Node *position=first_neuron;

    while(position)
    {
        position->SaveNeuronInformation(file);
        position=position->Next;
    }
}

void Layer::SaveConnection(ofstream &file)
{
    Node *position=first_neuron;

    while(position)
    {
        position->SaveConnection(file);
        position=position->Next;
    }
}

void Layer::LoadLayerInformation(ifstream &file)
{
    Node *position=first_neuron;

    while(position)
    {
        position->LoadNeuronInformation(file);
        position=position->Next;
    }
}

void Layer::SetNodeType(int type)
{
    node_type=type;
}

void Layer::EstimateBias()
{
    Node *position=first_neuron;
    while(position)
    {
        position->EstimateBias();
        position=position->Next;
    }
}

void Layer::DisplayNeuronsOutput( )
{
    Node *position=first_neuron;
    while(position)
    {
        cout << position->GetActivation() << "\t ";
        position=position->Next;
    }
}

Input::Input()
{
    type=INPUT;
    node_type = LINEAR;
    name="Input";
    first_neuron = NULL;
}

Layer *Input::Clone()
{
    Layer *l=new Input();
    if(!l)
        throw new MemoryException("no clone made (input->clone)");
    for(Node *n=this->first_neuron; n!=NULL ; n=n->Next)
        l->Add(n->Clone(),false);
    return l;
}

void Input::SetActivation(float *ActualSignal)
{
    Node *position = first_neuron;
    int counter=0;

    while(position)
    {
        position->SetActivation(ActualSignal[counter]);
        position=position->Next;
        counter++;
    }
}

void Input::CalculateActivation()
{}

void Input::CreateInitConnection(Layer *)
{}

Hidden::Hidden()
{
    type=HIDDEN;
    node_type=MAGIC;
    name="Hidden";
    first_neuron = NULL;
}

void Hidden::CalculateActivation()
{
    Node *position=first_neuron;

    while(position)
    {
        position->Activation();
        position=position->Next;
    }
}


void Hidden::CreateInitConnection(Layer *l)
{
    Node *position=first_neuron;

    while(position)
    {
        position->AddLayerToConnectionTable(l->first_neuron);
        position=position->Next;
    }
}

Layer *Hidden::Clone()
{
    Layer *l=new Hidden();
    if(!l)
        throw new MemoryException("no clone made (hidden->clone)");
    for(Node *n=first_neuron; n!=NULL ; n=n->Next)
        l->Add(n->Clone(),false);
    return l;
}

Output::Output()
{
    type=OUTPUT;
    node_type=LINEAR;
    name="Output";
    first_neuron=NULL;
}

Layer *Output::Clone()
{
    Layer *l=new Output();
    if(!l)
        throw new MemoryException("no clone made (output->clone)");
    for(Node *n=this->first_neuron; n!=NULL ; n=n->Next)
        l->Add(n->Clone(),false);
    return l;
}


void Output::CalculateActivation()
{
    Node *position=first_neuron;
    while(position)
    {
        position->Activation();
        position=position->Next;
    }
}

void Output::CreateInitConnection(Layer *l)
{
    Node *position = first_neuron;

    while(position)
    {
        position->AddLayerToConnectionTable(l->first_neuron);
        position=position->Next;
    }
}

Node* Layer::FindNeuron(int n)
{
    Node* position=first_neuron;

    while(position && position->number!=n)
        position=position->Next;

    return position;
}

void Layer::FreezeAllNodesInLayer()
{
    Node *position=first_neuron;

    while(position)
    {
        position->FreezeNode();
        position=position->Next;
    }
}

void Layer::SetAllWeightsInLayer(float w)
{
    Node *position=first_neuron;

    while(position)
    {
        position->SetWeights(w);
        position=position->Next;
    }
}

void Output::SetBackSignal( float * backErrors )
{
    Node *position=first_neuron;
    while(position)
    {
        position->SetBackSignal(*backErrors);
        position=position->Next;
        backErrors++;
    }
}

Node * Layer::FindPreviousNode( Node * currentNode )
{
    Node *position=first_neuron;
    while(position)
    {
        if(position->Next == currentNode) return position;
        position=position->Next;
    }
    return NULL;
}

Node * Layer::FindLastNode( )
{
    return FindPreviousNode( NULL );
}

int Layer::GetNumberOfNullConnections(float epsilon)
{
    int nullConnections = 0;
    
    for(Node *position=first_neuron; position != NULL ; position = position->Next)
    {
        nullConnections += position->GetNumberOfNullConnections(epsilon);
    }
    return nullConnections;
}

void Layer::Add(Node * n, float * weight)
{
    AddNode(n);
    if(Previus)
    {
        Node *position=Previus->first_neuron;
        while(position)
        {
            n->AddNodeToConnectionTable(position,*weight++);
            position=position->Next;
        }
    } else throw new Exception("There is no previus layer\n");
    
    if(Next)
    {
        Node *position=Next->first_neuron;
        while(position)
        {
            position->AddNodeToConnectionTable(n,0);
            position=position->Next;
        }
    }


}



