#ifndef __LAYER_H
#define __LAYER_H

#include <iostream>
#include <stdlib.h>
#include <fstream>
#include "Node.h"
#include "exceptions.h"
#include "Parameters.h"

#define INPUT 1
#define HIDDEN 2
#define OUTPUT 3

class Layer
{
protected:
    int nr_of_neurons;
    char *name;
    int node_type;	// 1 - sigmoid , 2 - LinearNode , 3 - magic

public:
    Node  *first_neuron;	//poczatek listy neuronow w warstwie
    Layer *Next;			//nastepna warstwa
    Layer *Previus;			//poprzednia warstwa

    static int counter;
    int number;				//numer warstwy
    int type;		// 1 - input, 2 hidden, 3 - output

    Layer();
    void InitLayer();
    void AddNode(Node *neuron);			//dodanie neuronu do listy w warstwie, bez polaczen
    void Add(Node *neuron,bool connect=false);
    void Add(Node *neuron,float *weight);

    void DelAllNodes();

//     void Del(Node *neuron)
//     {
//         throw new Exception(" not implemented");
//     };

    virtual void CreateInitConnection(Layer *previus_layer)=0;	//polaczenie wszystkich neuronow w warstwie z neuronami w warstwie poprzedniej
    virtual Layer *Clone()=0;			// tworzy kopie warstwy zawierajaca takie same neurony
    void DelNode(Node *neuron);			//usuniecie neuronu z warstwy
    void DelAllConnections();			//usuniecie wszystkich polaczen z poprzednia warstwa
    void FreezeAllNodesInLayer();
    void SetAllWeightsInLayer(float w);

    void NodeMemory(int size);
    int GetNrOfNeurons()
    {
        return nr_of_neurons;
    }

    void CalculateDiff();
//    void backSignal(float *output,float l_factor=0.0,float reward_factor=0.0,float penalty_factor=0.0);
 //   void backSignal(float output,float l_factor=0.0,float reward_factor=0.0,float penalty_factor=0.0);
    void CalculateBackSignal();
//    void ChangeWeights(float learning_factor,float regularization=0.0);
    void ChangeWeights( const network_parameters & p );
 
    float GetActivation(int n);	//aktywacja neuronu o numerze n
    float* GetActivations();
    float *GetProjections( );
    int GetNumberOfNullConnections(float epsilon);

    void ChangeSlope(float);
    
    void SetSlope(float s);
    void SetNodeType(int type);		// na razie uwzgledniane tylko w warstwie ukrytej
    virtual void CalculateActivation()=0;
    void EstimateBias();

    Node* FindNeuron(int number);
    Node *FindPreviousNode(Node *currentNode);
    Node *FindLastNode();

    void DisplayWeights();
    void DisplayNeuronsOutput();
    
    void SaveLayer(ofstream &);
    void SaveConnection(ofstream &);
    void LoadLayerInformation(ifstream &);

    virtual ~Layer();

    friend ostream &operator<<(ostream &o,Layer &l)
    {
        //		if(l.type==HIDDEN)
        {
            //	o << l.name << "   layer " << l.number << " have " << l.nr_of_neurons << " units\n";
            //			if(l.type==INPUT)
            //				return o;
            for(Node *n=l.first_neuron;n!=NULL;n=n->Next)
                o << l.name << " " << *n;
        }
        return o;
    }
};

class Input: public Layer
{

public:
    Input();
    void CalculateActivation();
    void CreateInitConnection(Layer *l);
    void SetActivation(float *);
    Layer *Clone();
};

class Hidden: public Layer
{

public:

    Hidden();
    ~Hidden()
    {}
    ;

    void CalculateActivation();
    void CreateInitConnection(Layer *l);
    Layer *Clone();

};

class Output:public Layer
{

public:
    Output();
    void SetBackSignal(float *backErrors);
    void CalculateActivation();
    void CreateInitConnection(Layer *l);
    Layer *Clone();
};

#endif
