#define DEBUG 0
#include "Network.h"

int Layer::counter=0;
int Network::debug_level=0;
int compare(const void *a,const void *b);

int comp_func(int *x,int *y)
{
    if(*x>*y)
        return(-1);
    if(*x<*y)
        return(1);
    return(0);
}

Network::Network(Data *d,char *n)
{
    First=NULL;
    Last=NULL;
    nr_of_inputs=0;
    nr_of_outputs=0;
    number_of_layers=0;
    number_of_neurons=0;
    max_rules_number=0;
    Layer::counter=0;
    Node::counter=0;
    data=d;
    o_info = NULL;
    is_created=0;
    initial_number_of_hidden_neurons=1;

}

Network::Network(Network &k)
{
    First=NULL;
    Last=NULL;
    nr_of_inputs=0;
    nr_of_outputs=0;
    number_of_layers=0;
    number_of_neurons=0;
    max_rules_number=0;
    o_info = NULL;
    Layer::counter=0;
    Node::counter=0;
    data=k.data;
    is_created=0;
    initial_number_of_hidden_neurons=1;
//    model_name="Neural network";
    k.CopyNetworkArchitectureTo(this);
    k.CopyNetworkParametersTo(this);
}

Network::~Network()
{
    Clear();
}

void Network::AddLayer(Layer *type)
{
    Layer *position;

    if(!type)
        throw new Exception("no layer to add (AddLayer)");

    if(!First)
    {
        First=type;
        Last=First;
    }
    else
    {
        position=First;
        while(position->Next)
            position=position->Next;

        position->Next=type;
        position->Next->Previus=position;
        Last=position->Next;
    }
    number_of_layers++;
}


void Network::DelAllConnections()
{
    Layer *position=Last;
    while(position)
    {
        position->DelAllConnections();
        position=position->Previus;
    }
}

void Network::DelLayers()
{
    Layer *position;
    while(Last!=First)
    {
        position=Last;
        Last=Last->Previus;
        Last->Next=NULL;
        delete position;
        number_of_layers--;
    }
    position=Last;
    Last=First=NULL;
    delete position;
    number_of_layers--;
}

Node * Network::CreateNode( neuron_type type )
{
    Node *n = NULL;
    switch(type)
    {
        case linear :
            n = new LinearNode();
            break;
        case sigmoid :
            n = new Sigmoid();
            break;
        case ssigmoid :
            n = new Magic();
            break;
        case bicentral :
            n = new BiCentral();
            break;
        default :
            n = new LinearNode();
    }
    if(!n) throw new MemoryException("node not created");
    return n;
}

void Network::CreateNetwork( int inputs, int outputs, int hidden, int * hidden_neurons )
{
    Layer::counter=0;
    AddLayer(new Input());

    this->nr_of_inputs = inputs;
    this->nr_of_outputs = outputs;
    
    Node::counter=0;
    for(int i=0;i<inputs;i++)
        First->Add(CreateNode(linear),false);

    for(int i=0;i<hidden;i++)
    {
        Layer *l=new Hidden();
        AddLayer(l);

        Node::counter=0;
        int nr_n=0;
        if(hidden_neurons) nr_n=hidden_neurons[i];
        else nr_n=param.initial_nr_of_hidden_neurons;

        for(int k=0;k<nr_n;k++)
            l->Add(CreateNode(param.hidden),false);
    }

    //wyjsciowa
    AddLayer(new Output());

    Node::counter=0;
    for(int i=0;i<outputs;i++)
        Last->Add(CreateNode(param.output),false);

    if(o_info) delete [] o_info;
    o_info = new struct output_info[nr_of_outputs];
    if(!o_info) throw new MemoryException("in function CreateNetwork");
    
    this->CreateConnection();
    this->InitNetwork();
    this->is_created=1;
}

void Network::CreateNetwork(int number_of_hidden_layers,int *n_table)
{
    if(!data) throw new Exception("Dataset not loaded (CreateNetwork)");
    CreateNetwork(data->GetNumberOfFeatures(),data->GetNumberOfClasses(),number_of_hidden_layers,n_table);
}

void Network::CreateConnection()
{
    Layer *position,*remember;

    position=First;
    remember=NULL;
    while(position)
    {
        position->CreateInitConnection(remember);
        remember=position;
        position=position->Next;
    }
}

void Network::InitNetwork()
{
    time_t g_time;
    g_time=time(&g_time);
    unsigned int seed = rand();
    srand((unsigned int) g_time + seed);

    Layer *l=First->Next;
    while(l)
    {
        l->InitLayer();
        l=l->Next;
    }
    
}

//stara wersja - lepiej uzywac przeladowanego operatora <<
void Network::DisplayWeights()
{
    cout << *this;
}

void Network::CalculateActivation(float *ActualSignal)
{
    ((Input*)First)->SetActivation(ActualSignal);
    
    Layer *l=First->Next;
    while(l)
    {
        l->CalculateActivation();
        l=l->Next;
    }
}

float Network::GetActivation(int n)
{
    Node *position=Last->first_neuron;
    while(position && n)
    {
        position=position->Next;
        n--;
    }
    if(n!=0)
        throw new Exception("Nie ma takiego neuronu wyjsciowego (GetActivation)");
    return position->GetActivation();
}

/**
    Tworzenie sieci w oparcie o dane i
    uruchamianie odpowiedniego 
    algorytmu uczenia
*/
void Network::Run( Data * data )
{
    ConnectWithData(data);
    int inputs = data->GetNumberOfFeatures();
    int outputs = data->GetNumberOfClasses();
    
    int hiddenLayersCount = (init_param.initial_nr_of_hidden_neurons == 0) ? 0 : 1 ;
    CreateNetwork(inputs,outputs,hiddenLayersCount);
    
    
    EstimateBias(data);
    
    if(debug_level > 0)
    {
        cout << "\n  Siec zainicjowana:" << *this;
    }
    //Learning(1);
    BestOfNInitiatins(param.number_of_initiatializations);
}

/* wyznacza wagi i progi neuronow nie zmienia architektury sieci */
/* strategie uczenia:
    flag = 1    - gdy sredni blad maleje wolno to zmniejszamy wsp. uczenia i
                po kilku cyklach konczymy jesli nic sie nie zmienia
    flag = 2    - przerywamy gdy sredni blad wzrosnie
*/

void Network::Learning(int flag)
{
    unsigned long k=0;
    int counter_end=0;
    int test=1;
    float currentAverageError=0.0,lastAverageError=0.0;
    float currentError = 0.0;
    unsigned long nr_vectors=data->GetNumberOfVectors();

    Network *best_network=this->DuplicateNetwork();
    
    CalculateNetworkError();

    if(debug_level > 1 )
    {
        cout << "Initiation" << "\t";
        PrintError();
        cout << "\t";
        DisplayNeuronsProjection(First->Next,1);
        if (debug_level > 2) cout << *this;
        cout << endl;
    }

    float beg_error=t_info.ms_error;
    float error_the_best=t_info.ms_error;
    
    RandomID rand(nr_vectors);
    int vec_id=0;
    int average = param.average;
    int numberOfCycles = param.max_nr_of_iterations; 
    
    if(debug_level > 0)
    {
        cout << "\n Begin of learning \n" << endl;
    }
    
    while(test && k < numberOfCycles)
    {
        k++;
        if(!flag) test=0;

        rand.Resample();
        
        for(unsigned long i=0;i<nr_vectors;i++)
        {
            vec_id = rand.Next();
            PropagateSignalThroughNetwork(data->GetInput(vec_id),data->GetOutput(vec_id));
            ChangeWeights(param);
        }

        CalculateNetworkError();
        currentError = t_info.ms_error;
        currentAverageError+=currentError;

        if(debug_level > 1 )
        {
            cout << "cycle=" << k << "\t";
            PrintError();
            cout << "\t";
            DisplayNeuronsProjection(First->Next,1);
            if (debug_level > 2) cout << *this;
            cout << endl;
        }

        if(error_the_best>currentError)
        {
            error_the_best=currentError;
            best_network->CopyNetworkWeightsFrom(this);
            best_network->t_info=this->t_info;
        }
        

        if(k%average==0)
        {
            currentAverageError/=(float)average;

// #if (DEBUG > 1)
// 
//                 ((BinaryNetwork *)this)->DisplayMagicNeuronsProjection(1);
// #endif

            if(debug_level > 0)
            cout<<"   Avg. error="<< currentAverageError << " after " << k << "cycles\n";

            if(flag==1)
            {
    // if(MY_ABS(lastAverageError-currentAverageError)< 0.02 *lastAverageError) tak bylo do 08.01.2007
                if(MY_ABS(lastAverageError-currentAverageError)< (param.eps * MY_ABS(lastAverageError)))
                {
                    counter_end++;
                    param.learning_rate-=0.1*param.learning_rate;
                }
                else  counter_end=0;

                //  if(counter_end>5) bylo do 08.01.2007
                if(counter_end >= 2 ) test=0;
            }
            
            if(flag==2)
            {
                if(lastAverageError<currentAverageError && k!= average)
                    test = 0;

            }
            lastAverageError=currentAverageError;
            currentAverageError=0;
        }
    }

    CopyNetworkWeightsFrom(best_network);
    t_info=best_network->t_info;
    t_info.cycles=k;

    if(debug_level > 0)
    {
        cout << "\n End of learning \n" << endl;
        cout << " Number of cycles : "<< t_info.cycles << endl;
        cout << " Learning parameters: learn " << param.learning_rate
        << "   regularization " << param.force_zero << endl;
        cout << " Begin error  : " << beg_error << endl;
        cout << " Best error : "<< error_the_best << endl;
    }
    delete best_network;
}

int Network::NumberOfNeurons()
{
    int count=0,counter=0,c=0;
    Node *position=Last->Previus->first_neuron;

    while(position)
    {
        if(position->Frozen())
            counter++;
        else
            count++;
        c++;
        position=position->Next;
    }

    //  cerr<<"\n Liczba nauczonych="<<counter<<" Inne="<<count <<" zero="<<param.end_zero;
    return(c);
}

void Network::SetSlope(float s)
{
    Layer *l=First->Next;
    while(l)
    {
        l->SetSlope(s);
        l=l->Next;
    }
}

void Network::CalculateDiffActivations()
{
    Layer *l=Last;
    while(l->Previus)
    {
        l->CalculateDiff();
        l=l->Previus;
    }
}

float Network::CalculateNetworkError()
{
    //    int number_of_classes=data->GetNumberOfClasses();
    unsigned long nr_vectors=data->GetNumberOfVectors();
    float *input=NULL;
    float *output=NULL;

    t_info.Clear();
    for(int i=0;i<nr_of_outputs;i++)
        o_info[i].Clear();
    t_info.wektorow=data->GetNumberOfVectors();

    for(unsigned long i=0;i<nr_vectors;i++)
    {
        input=data->GetInput(i);
        output=data->GetOutput(i);
        CalculateActivation(input);
        float *activations = Last->GetActivations();
        
        for(int k=0;k<nr_of_outputs;k++)
        {
            float out_j     = output[k];
            float activ_j   = activations[k];
            float tmp       = out_j-activ_j;
            float mseError = tmp*tmp;
            
            t_info.ms_error += mseError*(1-param.lambda_factor*out_j)
                               - param.reward_factor*out_j*activ_j
                               + param.penalty_factor*(1-out_j)*activ_j;
            
            t_info.mseError += mseError;
            
            float bias = 0.5;
            //Jezeli neuron sie wzbudza a nie powinien to uznajemy
            //to za blad
            if(activ_j>bias && out_j==0)
                o_info[k].blad0++;

            //jezeli sie nie wzbudza a powienien to za niedouczenie
            if(activ_j<=bias && out_j==1)
                o_info[k].blad1++;

            //ile poprawnie jest klasyfikowanych
            if(activ_j>bias && out_j==1)
                o_info[k].poprawne1++;

            if(activ_j <=bias && out_j==0)
                o_info[k].poprawne0++;
        }
        delete [] activations;
    }

    //	t_info.accuracy=(float)(t_info.poprawne1+t_info.poprawne0)/(float)t_info.wektorow;
    return(t_info.ms_error);
}

void Network::SetInitParameters()
{
    First=NULL;
    Last=NULL;
    number_of_layers=0;
    number_of_neurons=0;
    max_rules_number=0;

    param.learning_rate=0.1;
    //    param.force_zero=0.000001;
    //    param.force_one=0.0000000001;
    param.force_zero=0.0;
    param.force_one=0.0;
    param.max_nr_of_iterations = 1000;
    if(param.end_zero==0)
        param.end_zero=0.01;
    param.average=50;
    param.slope=2;       // poczatkowe
    param.strategy_of_learning=grow;
    param.reward_factor=0.0;
    param.penalty_factor=0.0;
    param.lambda_factor=0.0;
    param.number_of_initiatializations=10;
}

void Network::SetLearningRate(float lr)
{
    param.learning_rate=lr;
}

void Network::SetNumberOfIterations(long max)
{
    param.max_nr_of_iterations = max;
}

void Network::SetRegularizationFactor(float lf)
{
    param.force_zero=lf;
}

void Network::ChangeParameters(int cycle)
{
    static float rem;
    static int counter=0;

    if(cycle==0)
    {
        counter=0;
        rem=t_info.ms_error;
    }
    else
    {
        if(t_info.ms_error>1.3*rem)//czyli blad zwiekszyl sie 100% lub tez
            counter++;     //blad nie zmniejsza sie wcale

        rem=t_info.ms_error;
    }

    //Jezeli wystepuja duze skoki bledu to
    //zmniejsz wartosc parametru uczacego

    if(counter==10)
    {
        cerr<<"\n LEARN="<<param.learning_rate;
        param.learning_rate-=0.3*param.learning_rate;
        if(param.force_zero<0.001)
            param.force_zero*=10;
        else
        {
            param.force_zero=0.0;
            param.force_one*=10;
        }

        ChangeSlope(2);
        cerr<<"\n Parameter="<<param.learning_rate;
        counter=0;
    }
}

void Network::ChangeSlope(float k)
{
    Layer *l=First;
    while(l)
    {
        l->ChangeSlope(k);
        l=l->Next;
    }
}

void Network::Save(ofstream &o)
{
    Layer *l=First;
    Node *position=NULL;
    int  counter=0;

    o << "Network" << "\n" << number_of_layers;

    l=First;
    while(l)
    {
        o << "\n" << l->type << "\t" << l->GetNrOfNeurons();
        l->SaveLayer(o);
        l=l->Next;
    }

    //I teraz zapisanie polaczen
    l=First;
    while(l)
    {
        l->SaveConnection(o);
        l=l->Next;
    }
}

void Network::Save(char *file_name)
{
    ofstream file_out;
    file_out.open(file_name);
    if(!file_out)
        throw new FileException(file_name);

    Save(file_out);
    file_out.close();
}

void Network::Load(ifstream &file)
{
    int nr_of_connections=0,type,counter=0,test,nr_of_layers=0;
    float weight=0.0;
    int exist=0;
    Layer *l=NULL;

    if(First!=NULL)
    {
        cout << "Siec istnieje\n";
        exist =1;
    }

    Layer::counter=0;
    Node::counter=0;

    char name[256];
    file.getline(name,256);

    file >> nr_of_layers;
    if(exist && nr_of_layers!=this->number_of_layers)
        throw new Exception("Niezgodna liczba warstw (Load)");

    Layer *new_l=NULL;
    Node *new_n=NULL;

    l=First;
    for(int i=0;i<nr_of_layers;i++)
    {
        file >> type;
        file >> counter;
        if(!exist)
        {
            switch(type)
            {
            case INPUT:
                new_l = new Input();
                break;
            case HIDDEN:
                new_l = new Hidden();
                break;
            case OUTPUT:
                new_l = new Output();
                break;
            default:
                throw new Exception("Nieznany typ warstwy (Load)");
            }
            if(!new_l)
                throw new MemoryException("no layer (Load)");
            AddLayer(new_l);

            for(int k=0;k<counter;k++)
            {
                int node_type;
                file >> node_type;

                switch(node_type)
                {
                case LINEAR:
                    new_n = new LinearNode();
                    break;
                case SIGMOID:
                    new_n = new Sigmoid();
                    break;
                case MAGIC:
                    new_n = new Magic();
                    break;
                default:
                    throw new Exception("Nieznany typ neuronu (Load)");
                }
                if(!new_n)
                    throw new MemoryException("no neuron (Load)");
                new_n->LoadNeuronInformation(file);
                new_l->Add(new_n,false);
            }
        }
        else
        {
            if(l->GetNrOfNeurons()!=counter ||l->type!=type)
                throw new Exception("Niewlasciwa liczba neuronow w warstwie (Load)");
            l=l->Next;
        }
    }

    l=First;
    while(l)
    {
        Node *remember=l->first_neuron;

        while(remember)
        {
            file >> nr_of_connections;   //ilosc polaczen
            for(int i=0;i<nr_of_connections;i++)
            {
                file >>counter;   //numer neuronu do polaczenia z bierzacym
                file >>weight;    //waga polaczenia
                test=0;
                //Teraz szukamy neuronu o tym numerze w sieci
                Layer *m=First;
                while(m)
                {
                    Node* position=m->FindNeuron(counter);
                    if(position)
                    {

                        //                        cerr << "polaczenie " << remember << " z " << position
                        //                          << "(" << counter << ") waga=" << weight << endl;
                        if(!exist)
                            remember->AddNodeToConnectionTable(position);
                        remember->SetWeight(position,weight);
                        test=1;
                        break;
                    }
                    m=m->Next;
                }
                if(!test)
                {
                    cerr<<"\n Numer="<<counter;
                    throw new Exception("nie wiem o co chodzi ");
                }
            }
            remember=remember->Next;
        }
        l=l->Next;
    }
}

void Network::Load(char *file_name)
{
    ifstream file_in;
    file_in.open(file_name);
    if(!file_in)
        throw new FileException("file_in");

    Load(file_in);
    file_in.close();

    cout << "\n plik wczytany";
}


// kopjuje wartosci wag pomiedzy sieciami - obie sieci musza miec identyczna architekture
void Network::CopyNetworkWeightsFrom(Network *net)
{
    Layer *l=First;
    Layer *net_l=net->First;
    while(l && net_l)
    {
        Node *n=l->first_neuron;
        Node *net_n=net_l->first_neuron;

        while(n && net_n)
        {
            n->CopyNode(net_n);
            Node::Connection *position=n->backwardConnections;
            Node::Connection *net_position=net_n->backwardConnections;
            while(position && net_position)
            {
                if(net_position->neuron->number != position->neuron->number)
                    throw new Exception("Somthing wrong with network architecture (CopyNetworkWeightsFrom)");
                position->Weights=net_position->Weights;
                position=position->Next;
                net_position=net_position->Next;
            }
            if(net_position || position)
                throw new Exception("Somthing wrong with network architecture (CopyNetworkWeightsFrom)");
            n=n->Next;
            net_n=net_n->Next;
        }
        if(net_n || n)
            throw new Exception("Somthing wrong with network architecture (CopyNetworkWeightsFrom)");
        l=l->Next;
        net_l=net_l->Next;
    }

    if(net_l || l)
        throw new Exception("Somthing wrong with network architecture (CopyNetworkWeightsFrom)");
}

void Network::PrintError()
{
    cout  << " E=" << t_info.ms_error << " MSE=" << t_info.mseError; // << " acc.=" << t_info.accuracy;
    for(int i=0;i<nr_of_outputs;i++)
        cout <<  "\t" << o_info[i].poprawne0 << "|" << o_info[i].blad0 << "|" << o_info[i].blad1 << "|" << o_info[i].poprawne1;
    //	cout << endl;
}

/* przekopiowuje wagi i wszystkie inne informacje dotyczace neuronow
siec z ktorej kopiujemy musi miec identyczna architekture */

/*
void Network::CopyNetworkWeightsFrom(Network *n)
{
BinaryNetwork *net=(BinaryNetwork*)n;
output_number=net->output_number;
    Layer *l=First;
    Layer *net_l=net->First;
    while(l && net_l)
    {
        Node *n=l->first_neuron;
        Node *net_n=net_l->first_neuron;
 
        while(n && net_n)
        {
            n->CopyNode(net_n);
            Node::Connection *position=n->backwardConnections;
            Node::Connection *net_position=net_n->backwardConnections;
            while(position && net_position)
            {
                if(net_position->neuron->number != position->neuron->number)
                    Error("Somthing wrong with network architecture\n");
                position->Weights=net_position->Weights;
                position=position->Next;
                net_position=net_position->Next;
            }
            if(net_position || position) Error("Somthing wrong with network architecture\n");
            n=n->Next;
            net_n=net_n->Next;
        }
        if(net_n || n) Error("Somthing wrong with network architecture\n");
        l=l->Next;
        net_l=net_l->Next;
    }
 
    if(net_l || l) Error("Somthing wrong with network architecture\n");
}
*/


void Network::CopyNetworkParametersTo(Network *net)
{
    net->param=param;
    net->init_param = init_param;
    net->nr_of_inputs=nr_of_inputs;
    net->nr_of_outputs=nr_of_outputs;
    net->max_rules_number=max_rules_number;
    net->number_of_layers=number_of_layers;
    net->t_info=t_info;

    if(net->o_info) delete [] net->o_info;
    net->o_info = new struct output_info[nr_of_outputs];
    if(!net->o_info) throw new MemoryException(" o_info (CopyNetworkWeightsFrom)");

    if(o_info)    for(int i=0;i<net->nr_of_outputs;i++)    net->o_info[i]=o_info[i];
}


void Network::CopyNetworkArchitectureTo(Network *net)
{
    int nol=Layer::counter;
    int non=Node::counter;

    Layer::counter=0;
    for(Layer *l=First;l!=NULL;l=l->Next)
    {
        Node::counter=0;
        net->AddLayer(l->Clone());
    }

    Layer::counter=nol;
    Node::counter=non;

    //odtworzenie polaczen
    for(Layer *ll=First,*nl=net->First;ll!=NULL ;ll=ll->Next,nl=nl->Next)
    {
        for(Node *n=ll->first_neuron,*nn=nl->first_neuron;n!=NULL;n=n->Next,nn=nn->Next)
        {
            Node::Connection *position=n->backwardConnections;
            while(position)
            {
                int number=position->neuron->number;
                float weight=position->Weights;
                Node* tmp=nl->Previus->FindNeuron(number);
                if(!tmp)
                    throw new Exception("Can't find connection (CopyNetworkArchitectureTo)");
                nn->AddNodeToConnectionTable(tmp,weight);
                position=position->Next;
            }
        }
    }
    net->is_created=1;
}

Network* Network::DuplicateNetwork()
{
    Network *net = new Network(data);
    if(!net) return net;

    this->CopyNetworkArchitectureTo(net);
    this->CopyNetworkParametersTo(net);
    return net;
}

void Network::BestOfNInitiatins(int n)
{
    Network *net2 = this->DuplicateNetwork();
    net2->data = this->data;
    float error_the_best=0.0;

    struct network_parameters init_param = param;

    for(int i=0;i<n;i++)
    {
        if(debug_level>0) cout << "\n Network nr " << i << endl;
        
        net2->init_param = init_param;
        net2->InitNetwork();
        net2->SetSlope( init_param.slope );
        net2->EstimateBias( data );
        
        net2->Learning(param.strategy_of_learning);

        if(net2->t_info.ms_error < error_the_best || i==0)
        {
            error_the_best=net2->t_info.ms_error;
            *this=*net2;
        }
        if(debug_level > 0)
        {
            cout << " Network nr=" << i << " MSE error " << net2->t_info.ms_error << endl;
        }
    }
    delete net2;
}


unsigned long Network::GetNumberOfWrongClasified(int k)
{
    return o_info[k].blad0+o_info[k].blad1;
}

void Network::EstimateBias( Data * d )
{
    long vectorsCount=d->GetNumberOfVectors();
    for(long i=0;i<vectorsCount;i++)
    {
        CalculateActivation(d->GetInput(i));
        Layer *position=First->Next;
        while(position)
        {
            position->EstimateBias();
            position=position->Next;
        }
    }
}

int compare(const void *a,const void *b)
{
    struct projection *ap=(struct projection *)a;
    struct projection *bp=(struct projection *)b;
    if(ap->activation > bp->activation)
        return 1;
    return -1;
}

void Network::PropagateSignalThroughNetwork(float *input,float *output)
{
    CalculateActivation(input); // ustala aktywacje neuronow
    CalculateDiffActivations();           // sygnal zwrotny diffActivation
    CalculateBackSignal(output);
}

void Network::CalculateBackSignal(float *outputs)
{
    float l_factor = param.lambda_factor;
    float reward_factor = param.reward_factor;
    float penalty_factor = param.penalty_factor;
    
    float *activations = Last->GetActivations();
    float aa=activations[0];
    float bb = activations[1];

    for(int i=0;i<nr_of_outputs;i++)
    {
        activations[i] = (activations[i]-outputs[i])*(1.0-l_factor)
                         - reward_factor * outputs[i] * activations[i]
                         + penalty_factor * (1.0-outputs[i]) * activations[i];        
    
    }
    float a=activations[0];
    float b = activations[1];
    
    ((Output*)Last)->SetBackSignal(activations);
    
    Layer *l=Last->Previus;
    while(l)
    {
        l->CalculateBackSignal();
        l=l->Previus;
    }
    delete [] activations;
}

void Network::ChangeWeights( const network_parameters & p )
{
    Layer *l=Last;
    while(l)
    {
        l->ChangeWeights(p);
        l=l->Previus;
    }
}

void Network::SetRewardFactor(float r_factor)
{
    param.reward_factor=r_factor;
}

void Network::SetPenaltyFactor(float p_factor)
{
    param.penalty_factor=p_factor;
}

void Network::SetLFactor(float l_factor)
{
    param.lambda_factor=l_factor;
}

void Network::SetNrOfInitiations(int n)
{
    param.number_of_initiatializations=n;
}

void Network::Configure(Parameters &p)
{
    param = init_param = (network_parameters)p;
    debug_level = p.debug_level;
}

Label Network::Test(float *v)
{

    float max=0;
    int index=0;
    CalculateActivation(v);

    for(int i=0;i<nr_of_outputs;i++)
    {
        float output=GetActivation(i);
        if(output>max)
        {
            max=output;
            index=i;
        }
    }
    throw new Exception("poprawic");
    return 0;
//    return data->class_labels[index];
}

void Network::Reset()
{
    param = init_param;
    if(o_info) delete [] o_info;
    o_info = NULL;

/*    for(int i=0;i<nr_of_outputs;i++)
        o_info[i].Clear();*/
    t_info.Clear();
}

void Network::Clear( )
{
    DelAllConnections();
    DelLayers();
    if(o_info) delete [] o_info;
    o_info = NULL;
    t_info.Clear();
    is_created = 0 ;
}

void Network::CopyNetwork( Network & net )
{
    Clear();
    net.CopyNetworkArchitectureTo(this);
    *this = net;
}

void Network::ConnectWithData( Data * d )
{
    data = d;
}

int Network::GetNumberOfNullConnections(float epsilon)
{
    int nullConnections = 0;
    for(Layer *l=First;l!=NULL;l=l->Next)
    {
        nullConnections += l->GetNumberOfNullConnections(epsilon);
    }

    return nullConnections;
}
