#ifndef __NETWORK_H
#define __NETWORK_H

#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <vector>
#include "Model.h"
#include "Layer.h"
#include "Data.h"
#include "exceptions.h"
#include "Parameters.h"
#include "RandomID.h"
#include "BitVector.h"

using namespace std;

#define MY_ABS(x)  ((x)<0? -(x):(x))


struct projection
{
        int input_nr;
        float activation;
        float output;
};

int compare(const void *a,const void *b);
	
	
struct output_info
{
		unsigned long blad1;		//ile razy neuron sie nie nauczyl 1 (niedouczenie)
		unsigned long blad0;		//ile razy neuron sie nie nauczyl 0 (przeuczenie)
		unsigned long poprawne1;	//liczba poprawnych aktywacji dla 1 na wyjsciu
		unsigned long poprawne0;	//liczba poprawnych aktywacji dla 0 na wyjsciu
		void Clear()
		{
			blad1=blad0=poprawne1=poprawne0=0;
		}
		output_info():blad1(0),blad0(0),poprawne1(0),poprawne0(0){};
};

	
struct training_info
{
		float accuracy;				//(poprawne1+poprawne2)/wektorow
		float ms_error;				//blaad MSE
		float mseError;
		unsigned long wektorow;		//liczba wszystkich wektorow uczacych
		unsigned long cycles;
		void Clear()
		{
			accuracy=ms_error=mseError=0.0;
			//cycles=0;
		}
		training_info():accuracy(0.0),wektorow(0),ms_error(0.0),cycles(0),mseError(0.0){};
};

class Network //: public Model
{
    int max_rules_number;
protected:
    static int debug_level;
    long nr_of_inputs;
    long nr_of_outputs;
    int initial_number_of_hidden_neurons;

    struct network_parameters param;
    struct network_parameters init_param;

    Layer *First;
    Layer *Last;
    Data *data;
	
//	Parameters *init_params;
    void AddLayer(Layer *type);
    void DelAllConnections();
    void DelLayers();
    void CreateConnection();
    
    void SetInitParameters();
    void PropagateSignalThroughNetwork(float *input,float *output);
//    void backSignal(float *output,float l_factor=0.0,float reward_factor=0.0,float penalty_factor=0.0);
//    void ChangeWeights(float learning_factor=0.5, float regularization=0.0);
    void ChangeWeights( const network_parameters & p );
    virtual void CalculateBackSignal(float *output);          
    Node *CreateNode(neuron_type type);
    
    void CalculateDiffActivations();

    void CopyNetworkWeightsFrom(Network *net);
    void CopyNetworkArchitectureTo(Network *net);
    void CopyNetworkParametersTo(Network *net);

public:
    int is_created;				//czy siec jest juz stworzona
    virtual void CreateNetwork(int inputs,int outputs,int hidden = 1 , int *hidden_neurons = NULL);
    void CopyNetwork(Network &net);
    struct training_info t_info;
    struct output_info *o_info;

    int number_of_neurons;
    int number_of_layers;

    Network(Data *data=NULL,char *name=NULL);
    Network(Network &k);
    virtual ~Network();

    //architektura
    void InitNetwork();
    void EstimateBias(Data *d);
    virtual void CreateNetwork(int hidden_layers,int *neurons_table=NULL);
    virtual Network* DuplicateNetwork();
    virtual void SetDebugLevel(int l){debug_level = l;}
    
    Label Test(float *vector);
    void Reset();
    void Learning(int strategy);
    void BestOfNInitiatins(int n);
    virtual void Run(Data *data);
    
    void ConnectWithData(Data *d);
        
    void CalculateActivation(float *);
    virtual float CalculateNetworkError();

    //informacje o sieci
    float GetActivation(int nr_of_output_neuron);
    unsigned long GetNumberOfWrongClasified(int output_number);
    int NumberOfNeurons();
    void Clear();
    int GetNumberOfNullConnections(float epsilon=1e-6);
    //ustawienie parametrow
    void Configure(Parameters &p);
    
    void SetRewardFactor(float r_factor);
    void SetPenaltyFactor(float p_factor);
    void SetLFactor(float l_factor);
    void SetLearningRate(float lr);
    void SetNumberOfIterations(long max);
    void SetRegularizationFactor(float lf);
    void SetSlope(float slope);
    void SetNrOfInitiations(int n);

    void ChangeParameters(int cycle);
    void ChangeSlope(float value);

    //operacje we-wy
    void Save(char *file_name);
    void Load(char *file_name);
    virtual void Save(ofstream &out);
    virtual void Load(ifstream &in);
    
    void DisplayWeights();
    void PrintError();
    virtual void DisplayNeuronsProjection(Layer *l,int type=0)
    {
       // throw new Exception("DisplayNeuronProjection not implemented");
        /* not implemented */
    };

    Network& operator=(Network &n)
    {
        CopyNetworkWeightsFrom(&n);	
        n.CopyNetworkParametersTo(this);
        return *this;
    }

    friend ostream& operator<<(ostream &o,Network &net)
    {
        o << "\n ------ Network architecture ---------------------------- \n\n";
        for(Layer *l=net.First;l!=NULL;l=l->Next)
        if(l->type != INPUT) 
            o << *l;
        return o;
    }
};



#endif
