#include "Node.h"

float function_sigm(float x)
{
    if(x>100) return 1;
    if(x<-100) return 0;
    return 1.0/(1.0+exp(-x));
}

Node::Node()
{
    Next=NULL;
    backwardConnections=NULL;
    forwardConnections=NULL;
    activation=0;
    diffActivation=0;
    bias=0;
    slope=1;
    freeze=0;
    Features=0;
    counter++;
    number=counter;
    end_zero=0;
    type=0;
    signal_output=0.0;
}

Node::~Node()
{
      DelConnectionTable();
}

void Node::DelConnectionTable()
{
    Connection *position;
    while(backwardConnections)
    {
        position=backwardConnections;
        backwardConnections=backwardConnections->Next;
        delete position;
    }
    Connection_forward *position_f;
    while(forwardConnections)
    {
        position_f=forwardConnections;
        forwardConnections=forwardConnections->Next;
        delete position_f;
    }
}

void Node::AddNodeToConnectionTable(Node *n)
{
    Connection *position;
    Connection_forward *forward;

    if(!backwardConnections)
    {
        backwardConnections=new Connection;
        if(!backwardConnections)
            throw new MemoryException("connection table not created (AddNodeToConnectionTable)");
        backwardConnections->neuron=n;
        position=backwardConnections;
        backwardConnections->Next=NULL;
    }
    else
    {
        position=backwardConnections;
        while(position->Next)
            position=position->Next;

        position->Next=new Connection;
        if(!position->Next)
            throw new MemoryException("connection table not created (AddNodeToConnectionTable)");

        position->Next->neuron=n;
        position=position->Next;
        position->Next=NULL;
    }

    if(!n->forwardConnections)
    {
        n->forwardConnections=new Connection_forward;
        if(!n->forwardConnections)
            throw new MemoryException("forward connection table not created (AddNodeToConnectionTable)");
        n->forwardConnections->neuron=this;
        n->forwardConnections->Weights=&position->Weights;
        n->forwardConnections->Next=NULL;
    }
    else
    {
        forward=n->forwardConnections;

        while(forward->Next)
            forward=forward->Next;

        forward->Next=new Connection_forward;
        if(!forward->Next)
            throw new MemoryException("forward connection table not created (AddNodeToConnectionTable)");

        forward->Next->neuron=this;
        forward->Next->Weights=&position->Weights;
        forward->Next->Next=NULL;
    }
}

void Node::AddNodeToConnectionTable(Node *n, float weight)
{
    Connection *position;
    Connection_forward *forward;

    if(!backwardConnections)
    {
        backwardConnections=new Connection;
        if(!backwardConnections)
            throw new MemoryException("connection table not created (AddNodeToConnectionTable)");
        backwardConnections->neuron=n;
        backwardConnections->Weights=weight;
        position=backwardConnections;
        backwardConnections->Next=NULL;
    }
    else
    {
        position=backwardConnections;
        while(position->Next)
            position=position->Next;

        position->Next=new Connection;
        if(!position->Next)
            throw new MemoryException("connection table not created (AddNodeToConnectionTable)");

        position->Next->neuron=n;
        position->Next->Weights=weight;
        position=position->Next;
        position->Next=NULL;
    }

    if(!n->forwardConnections)
    {
        n->forwardConnections=new Connection_forward;
        if(!n->forwardConnections)
            throw new MemoryException("forward connection table not created (AddNodeToConnectionTable)");
        n->forwardConnections->neuron=this;
        n->forwardConnections->Weights=&position->Weights;
        n->forwardConnections->Next=NULL;
    }
    else
    {
        forward=n->forwardConnections;

        while(forward->Next)
            forward=forward->Next;

        forward->Next=new Connection_forward;
        if(!forward->Next)
            throw new MemoryException("forward connection table not created (AddNodeToConnectionTable)");

        forward->Next->neuron=this;
        forward->Next->Weights=&position->Weights;
        forward->Next->Next=NULL;
    }
}

void Node::DelNodeFromConnectionTable(Node *n)
{
    Connection *position,*remember;
    Connection_forward *pos,*rem;

    if(backwardConnections)
    {
        position=backwardConnections;
        remember=NULL;
        while(position->neuron!=n || position->Next!=NULL)
        {
            remember=position;
            position=position->Next;
        }

        if(remember)
        {
            remember->Next=position->Next;
            delete position;
        }
        else
        {
            backwardConnections=position->Next;
            delete position;
        }
    }

    if(forwardConnections)
    {
        pos=forwardConnections;
        rem=NULL;

        while(pos->neuron!=n || pos->Next!=NULL)
        {
            rem=pos;
            pos=pos->Next;
        }

        if(rem)
        {
            rem->Next=pos->Next;
            delete pos;
        }
        else
        {
            forwardConnections=pos->Next;
            delete pos;
        }
    }
}

void Node::AddLayerToConnectionTable(Node *n)
{
    Node *position=n;

    while(position)
    {
        AddNodeToConnectionTable(position);
        position=position->Next;
    }
}

void Node::DelLayerFromConnectionTable(Node *n)
{
    Node *position=n;

    while(position)
    {
        DelNodeFromConnectionTable(position);
        position=position->Next;
    }
}

void Node::InitNode()
{
    Connection *position=backwardConnections;

    if(!freeze)
    {
        while(position)
        {
            position->Weights=(float)(rand()%200-100)/100;
            position=position->Next;
        }
        bias=(float)(rand()%200-100)/100;
    }
}

/**
* Set value of single weight.
*	Set waight value between this node and node from perwavius layer selected by position pointer.
* @param position pointer to connected node from previous layel
* @param value value of weight to set
*/
void Node::SetWeight(Node *position,float value)
{
    Connection *remember = backwardConnections;

    while(remember)
    {
        if(remember->neuron==position)
        {
            remember->Weights=value;
            break;
        }
        remember=remember->Next;
    }
}

void Node::SetWeights(float value)
{
    Connection *remember = backwardConnections;

    while(remember)
    {
        remember->Weights=value;
        remember=remember->Next;
    }
}


int Node::Frozen()
{
    return(freeze);
}

//Sprawdzenie czy wszsytkie wezly z nim polaczone
//sa zamrozone
int Node::CheckIfFrozen()
{
    Connection *position;
    int test;

    test=1;

    position=backwardConnections;
    while(position)
    {
        if(!position->neuron->Frozen())
        {
            test=0;
            break;
        }
        position=position->Next;
    }
    return(test);
}

void Node::FreezeNode()
{
    freeze=1;
}

void Node::UnFreezeNode()
{
    freeze=0;
}

void Node::CalculateBackSignal()
{
    Connection_forward *position=forwardConnections;
    backSignal=0.0;

    while(position)
    {
        backSignal+=(*position->Weights)*position->neuron->slope*position->neuron->backSignal;
        position=position->Next;
    }
    backSignal*=diffActivation;
}
/*
void Node::ChangeWeights(float learning_rate,float regularization)
{
    if(!freeze)
    {
        Connection *position=backwardConnections;
        float delta = learning_rate * backSignal ;
        float rslope=slope;
        while(position)
        {
            position->Weights-= delta * position->neuron->activation * rslope
                                +regularization*position->Weights;
            position=position->Next;
        }

        slope -= delta * (projection-bias);            //        xxxx;
        bias += delta*rslope;
    }
}*/




//stara wersja - uzywac przeladowanego operatora <<
void Node::DisplayWeights()
{
    cout << *this;
}

ostream& Node::DisplayWeights(ostream &o)
{
    Connection *position=backwardConnections;

    o << "   n " << number << " " << name ;
    if (Frozen())
        o << " *Frozzen* ";

    if(!position)
    {
        o << " - no connection\n";
        return o;
    }

    o << "  w [ ";
    while(position)
    {
        o << std::right << position->Weights
        //<< " (" << position->neuron->number<< ")  "
        << " ";
        position=position->Next;
    }
    o << "]   b [ " << bias << " ]   s [ " << slope << " ]" << endl;

    return o;
}

void Node::SaveNeuronInformation(ofstream &file)
{
    //pierwsza liczba musi byc typ neuronu
    file << "\n" << type << "\n" << number <<"\t"<< freeze <<"\t"<< Features <<"\t"<< slope << "\t" << bias ;
}

void Node::SaveConnection(ofstream &file)
{
    Connection *position=backwardConnections;
    int coun=0;

    while(position)
    {
        coun++;
        position=position->Next;
    }

    file <<"\n"<<coun;
    position=backwardConnections;
    while(position)
    {
        file <<"\n"<<position->neuron->number;
        file <<"\t"<<position->Weights;
        position=position->Next;
    }
}

void Node::LoadNeuronInformation(ifstream &file)
{
    file >>number;
    file >>freeze;
    file >>Features;
    file >>slope;
    file >>bias;
}

void Node::CopyNode(Node *n)
{
    if(!n)
        throw new Exception("Node error!!! (CopyNode)");
    number=n->number;
    bias=n->bias;
    freeze=n->freeze;
    Features=n->Features;
    end_zero=n->end_zero;
    type=n->type;
    slope=n->slope;
}

void Node::CalculateProjection( )
{
    Connection *position=backwardConnections;
    projection = 0.0 ;
    while(position)
    {
        projection+=position->Weights*position->neuron->GetActivation();
        position=position->Next;
    }
}

void Sigmoid::Activation()
{
    CalculateProjection();
    float sum=slope*(projection-bias);
    activation=function_sigm(sum);
}

void Sigmoid::ChangeWeights(const network_parameters  &p )
{
    if(!freeze)
    {
        float lr = p.learning_rate;
        float re = p.regularization;
        float delta = lr * backSignal ;
        float rslope=slope;
        float ds = delta * slope;
        
        Connection *position=backwardConnections;
        while(position)
        {
            position->Weights-= ds * position->neuron->GetActivation() 
                                + re * position->Weights;
            position=position->Next;
        }

       // float d2= delta * (projection-bias);
        //slope -= d2;
        bias += ds;
    }

}

Node *Sigmoid::Clone()
{
    Node *n=this->CreateNode();
    n->CopyNode(this);
    return n;
}

void Sigmoid::DiffActivation()
{
    diffActivation=activation*(1-activation);
}

void Sigmoid::EstimateBias( )
{
    if(freeze) return;
    if(y_min==0.0 && y_max==0.0)  y_min=y_max=projection;
    if(y_min > projection)        y_min = projection;
    if(y_max < projection)        y_max = projection;

    float yd=y_max-y_min;

    bias=y_min+0.5*yd;
}

void LinearNode::Activation()
{
    CalculateProjection();
    activation=slope * (projection - bias);
}

void LinearNode::ChangeWeights(const network_parameters  &p )
{
    if(!freeze)
    {
        float lr = p.learning_rate;
        float re = p.regularization;
        float delta = lr * backSignal ;
        float rslope=slope;
        float ds = delta * slope;
        
        Connection *position=backwardConnections;
        while(position)
        {
            position->Weights-= ds * position->neuron->GetActivation() 
                                + re * position->Weights;
            position=position->Next;
        }

        float d2= delta * (projection-bias);
        //slope -= d2;
        bias += ds;
    }
}

Node *LinearNode::Clone()
{
    Node *n = CreateNode();
    n->CopyNode(this);
    return n;
}

void LinearNode::DiffActivation()
{
    diffActivation=1.0;
}

void LinearNode::EstimateBias( )
{
    if(freeze) return;
    if(y_min==0.0 && y_max==0.0)  y_min=y_max=projection;
    if(y_min > projection)        y_min = projection;
    if(y_max < projection)        y_max = projection;

    float yd=y_max-y_min;

    bias=y_min+0.5*yd;
}

ostream& LinearNode::DisplayWeights(ostream &o)
{
    Connection *position=backwardConnections;

    o << "   n " << number << " " << name ;
    if (Frozen())
        o << " *Frozzen* ";

    if(!position)
    {
        o << " - no connection\n";
        return o;
    }

    o << "  w [ ";
    while(position)
    {
        o << std::right << position->Weights
        //<< " (" << position->neuron->number<< ")  "
        << " ";
        position=position->Next;
    }
    o << " ]   b [ " << bias << " ]   s [ " << slope << " ] " <<  endl;
    return o;
}

Magic::Magic()
{
    name="Magic";
    type=MAGIC;
    activ1=activ2=0.0;
    bias2=0;
    projection = 0;
    diffActiv1=diffActiv2=0;
    backSignal1=0.0;
    backSignal2=0.0;
    lc=1.0;         //defoultowo szukamy klastra z 1-nek,
    y_max=y_min=0.0;
}

Magic::Magic(int learn_class)
{
    name="Magic";
    type=MAGIC;
    activ1=activ2=0.0;
    bias2=0;
    diffActiv1=diffActiv2=0;
    backSignal1=0.0;
    backSignal2=0.0;
    if(learn_class == 1)
        lc=1.0;
    else
        lc=0.0;
}

void Magic::Activation()
{
    CalculateProjection();
    
    float sum1=slope*(projection-bias);
    float sum2=slope*(projection-bias2);

    activ1=function_sigm(sum1);
    activ2=function_sigm(sum2);
    activation=activ1-activ2;
}

void Magic::DiffActivation()
{
    diffActiv1=activ1*(1-activ1);
    diffActiv2=activ2*(1-activ2);
    diffActivation=diffActiv1-diffActiv2;
}

// void Magic::ChangeWeights(float learning_rate,float regularization)
// {
//     if(!freeze)
//     {
//         float rslope=slope;
//         float delta1 = learning_rate * backSignal1 ;
//         float delta2 = learning_rate * backSignal2 ;
//         float delta = learning_rate * backSignal;
//         //	if (delta < 0 )
//         float ds = (delta1 * (projection-bias) - delta2 * (projection-Bias2));
//         //     if (ds < 0 ) slope-=ds;
//         //     else slope-= 0.3*ds;
// 
//         Connection *position=backwardConnections;
//         while(position)
//         {
//             position->Weights-= delta * rslope * position->neuron->GetActivation()
//                                 //                         -      -r_factor*actual_output*delta
//                                 +regularization*position->Weights;
//             position=position->Next;
//         }
// 
//         bias+= delta1 * rslope;
//         Bias2-= delta2 * rslope;
//     }
// }


void Magic::ChangeWeights( const network_parameters & p )
{
    if(!freeze)
    {
        float rslope=slope;
        float lr = p.learning_rate;
        float re = p.regularization;
        float delta1 = lr * backSignal1 ;
        float delta2 = lr * backSignal2 ;
        float ds = (delta1 * (projection-bias) - delta2 * (projection-bias2));
        float delta  = lr * backSignal * slope;
      
        if(p.slope_learning == 1 || (p.slope_learning == 2 && ds < 0))
            slope-= ds;
        if(p.slope_learning == 2 && ds > 0 )
           slope-= 0.5 * ds;

        Connection *position=backwardConnections;
        while(position)
        {
            position->Weights-= delta * position->neuron->GetActivation()
                                + re * position->Weights;
            position=position->Next;
        }

        bias+= delta1 * rslope;
        bias2-= delta2 * rslope;
    }
}

void Magic::CalculateBackSignal()
{
    Connection_forward *position=forwardConnections;
    float Wsignal = 0.0;

    while(position)
    {
        Wsignal+=(*position->Weights) * position->neuron->GetSlope() * position->neuron->backSignal;
        position=position->Next;
    }

    backSignal1=diffActiv1*Wsignal;
    backSignal2=diffActiv2*Wsignal;
    backSignal=diffActivation*Wsignal;
}
//stara wersja - uzywac przeladowanego operatora <<
void Magic::DisplayWeights()
{
    cout << *this;
}

void Magic::SaveNeuronInformation(ofstream &o)
{
    Node::SaveNeuronInformation(o);
    o << "\t" << bias2;
}

void Magic::LoadNeuronInformation(ifstream &i)
{
    Node::LoadNeuronInformation(i);
    i >> bias2;
}

void Magic::InitNode()
{
    Node::InitNode();
    if(!freeze)
        bias2=(float)(rand()%200-100)/100;
}

void Magic::CopyNode(Node *n)
{
    Node::CopyNode(n);
    bias2=((Magic *)n)->bias2;
}

Node *Magic::Clone()
{
    Node *n=CreateNode();
    n->CopyNode(this);
    return n;
}

ostream& Magic::DisplayWeights(ostream &o)
{
    Node::Connection *position=backwardConnections;

    o << "   n " << number << " " << name ;
    if (Frozen())
        o << " *Frozzen* ";

    if(!position)
    {
        o << " - no connection \n";
        return o;
    }

    o << "   w [ ";
    while(position)
    {
        o << std::right << position->Weights
        //<< " (" << position->neuron->number<< ")  "
        << " ";
        position=position->Next;
    }
    o << "]   b [" << bias << " " << bias2 << " ]   s [" << slope << " ] " << endl ;

    return o;
}

void Magic::EstimateBias()
{
    if(freeze)  return;
    if(y_min==0.0 && y_max==0.0) y_min=y_max=projection;
    if(y_min > projection)       y_min = projection;
    if(y_max < projection)       y_max = projection;

    float yd=y_max-y_min;

    bias=y_min + 0.25 * yd;
    bias2=y_min + 0.75 * yd;
}

BiCentral::BiCentral()
{
    name="BiCentral";
    type=BICENTRAL;
    activ1=activ2=0.0;
    diffActiv1=diffActiv2=0;
    backSignal1=0.0;
    backSignal2=0.0;
    y_max=y_min=0.0;
    expBias = 0.0;
}

void BiCentral::Activation()
{
    CalculateProjection();
    
    float sum1=slope*(projection-center+bias);
    float sum2=slope*(projection-center-bias);

    activ1=function_sigm(sum1);
    activ2=function_sigm(sum2);
    activation=activ1*(1-activ2);
}

void BiCentral::DiffActivation()
{
    diffActiv1=activ1*(1-activ1)*(1-activ2);
    diffActiv2=activ2*(1-activ2)*activ1;
    diffActivation=diffActiv1-diffActiv2;
}

// void BiCentral::ChangeWeights(float learning_rate,float regularization)
// {
// 
//     if(!freeze)
//     {
//         float rslope = slope;
//         float delta = backSignal1*(projection-center-bias)-backSignal2*(projection-center+bias);
//         //	if (delta < 0 )
//         slope-= learning_rate*delta;
//         //			else slope-= 0.5*learning_rate*delta;
// 
//         Connection *position=backwardConnections;
//         while(position)
//         {
//             delta=rslope*backSignal*position->neuron->GetActivation();
//             position->Weights-=learning_rate*delta
//                                +regularization*position->Weights;
//             position=position->Next;
//         }
// 
//         bias+=learning_rate*rslope*(diffActiv1+diffActiv2);
//         center-=learning_rate*rslope*( 1 /*no wlasnie */);
// 
//     }
// }

void BiCentral::ChangeWeights(const network_parameters &p)
{
    if(!freeze)
    {
        float lr = p.learning_rate;
        float re = p.regularization;
        float lrs = lr * slope;
        float delta = backSignal1*(projection-center+bias)-backSignal2*(projection-center-bias);

        if(p.slope_learning == 1 || (p.slope_learning == 2 && delta < 0))
            slope-= lr * delta;
        if(p.slope_learning == 2 && delta > 0 )
            slope-= 0.5 * lr * delta;
        
        float delta2 = lrs * backSignal;
        Connection *position=backwardConnections;
        while(position)
        {
            position->Weights-= delta2 * position->neuron->GetActivation()
                               + re * position->Weights;
            position=position->Next;
        }
        
        expBias -= lrs * (backSignal1 + backSignal2) * bias;
        bias = exp(expBias);
        center+=lrs * backSignal;
    }
}

void BiCentral::CalculateBackSignal()
{
    Connection_forward *position=forwardConnections;
    backSignal=0.0;
    backSignal1=0.0;
    backSignal2=0.0;
    float wsignal = 0.0;
    while(position)
    {
        wsignal+=(*position->Weights)*position->neuron->backSignal *position->neuron->GetSlope();
        position=position->Next;
    }
    backSignal1 = diffActiv1*wsignal;
    backSignal2 = diffActiv2*wsignal;
    backSignal  = diffActivation*wsignal;
}
//stara wersja - uzywac przeladowanego operatora <<

void BiCentral::SaveNeuronInformation(ofstream &o)
{
    Node::SaveNeuronInformation(o);
    o << "\t" << center;
}

void BiCentral::LoadNeuronInformation(ifstream &i)
{
    Node::LoadNeuronInformation(i);
    i >> center;
}

void BiCentral::InitNode()
{
    Node::InitNode();
    if(!freeze)
    {
        center=(float)(rand()%200-100)/100;
        bias = fabs((double)bias);
        expBias = log(bias);
    }        
}

void BiCentral::CopyNode(Node *n)
{
    Node::CopyNode(n);
    center=((BiCentral *)n)->center;
    expBias=((BiCentral *)n)->expBias;
}

Node *BiCentral::Clone()
{
    Node *n=CreateNode();
    n->CopyNode(this);
    return n;
}

ostream& BiCentral::DisplayWeights(ostream &o)
{
    Node::Connection *position=backwardConnections;

    o << "   n " << number << " " << name ;
    if (Frozen()) o << " *Frozzen* ";

    if(!position)
    {
        o << " - no connection \n";
        return o;
    }

    o << "   w [ ";
    while(position)
    {
        o << std::right << position->Weights
        //<< " (" << position->neuron->number<< ")  "
        << " ";
        position=position->Next;
    }
    o << "]   b [" << bias << "] c [ " << center << " ]   s [" << slope << " ] " << endl ;

    return o;
}

void BiCentral::EstimateBias()
{
    if(freeze)
        return;
    if(y_min==0.0 && y_max==0.0) y_min=y_max=projection;
    if(y_min > projection)       y_min = projection;
    if(y_max < projection)       y_max = projection;

    float yd=y_max-y_min;

    bias=0.25*yd;
    center = y_min + 0.5*yd;
    expBias = log(bias);
}

void Node::CopyWeightsFromNode( Node * n )
{
    Connection *srcConnection = n->backwardConnections;
    Connection *dstConnection = this->backwardConnections;
    
    while(srcConnection && dstConnection)
    {
        dstConnection->Weights = srcConnection->Weights;
        dstConnection = dstConnection->Next;
        srcConnection = srcConnection->Next;
    }
}

int Node::GetNumberOfNullConnections(float epsilon)
{
    int nullConnections = 0;
    for(Node::Connection *position=backwardConnections;position != NULL; position = position->Next)
    {
        if(fabs((double)position->Weights) < epsilon) nullConnections++;
    }
    return nullConnections;
}

float * Node::GetWeights()
{
    int nconn=0;
    for(Node::Connection *position=backwardConnections;position != NULL; position = position->Next)
        nconn++;
    float *w = new float[nconn];
    int i=0;
    for(Node::Connection *position=backwardConnections;position != NULL; position = position->Next)
    {
        w[i++]=position->Weights;
    }
    
    return w;
}




