#ifndef __NODE_H
#define __NODE_H

#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include "exceptions.h"
#include "Data.h"
#include "BinaryData.h"
#include "struct.h"
#include "Parameters.h"

#define SIGMOID 1
#define MAGIC   3
#define LINEAR  2
#define BICENTRAL 4

float function_sigm(float x);

class Node
{

protected:
    char *name;           /**< nazwa neuronu np. sigmoidalny */
    float activation;          /**< aktualna wartosc aktywacji Active = sigm(wx) */
    float projection;     //!< aktualna wartosc rzutu wx
    int type;				///< typ neuronu
    float  bias;                   ///< prog
    int freeze;				         ///< czy zamrozone wagi
    float diffActivation;               ///< aktualna wartosc pochodnej funkci transferu, np.  activation (1 - activation)
    float slope;                 ///< nachylenie  funkcji sigmoidalnej
       
    void CalculateProjection();

public:
    Node();

    Node *Next;				///< nastepny neuron w warstwie
    float backSignal;              ///< sygnal zwrotny (bledu) \sum{w_{i-1}BackSignal_{i-1}}*diffActivation
 
    float signal_output;    ///< OBECNIE nie uzywane -  ustawiane tylko dla neuronow wyjsciowych - ktore wyjscie ma byc brane pod uwage przy obliczniu bledu

    /**
    	polaczenie pomiedzy neuronami
    */
    struct Connection
    {
        Node *neuron;          ///< neuron polaczony w warstwie poprzedniej
        float Weights;         ///< wartosc wagi
        Connection *Next;      ///< nastepne polaczenie
    }
    *backwardConnections;		     ///< lista polaczen z neuronami z warstwy poprzedzajacej

    /**
    	polaczenia "w przod" - do neuronow w warstwie nastepnej
    */
    struct Connection_forward
    {
        Node *neuron;             ///< z tym neuronem laczyny
        float *Weights;           ///< wskaznik do wagi
        Connection_forward *Next; ///< nastepne polaczenie
    }
    *forwardConnections;		       ///< lista polaczen z neuronami w warstwie nastepnej

    //    float ActivationSignal; // suma aktywacji neuronow przylaczanych wagami = sum{Activ_i}
    int Features;
    int number;				        ///< numer neuronu w warstwie
    static int counter;		     ///< statyczny licznik zwiekszany przy tworzeniu nowego neuronu
    float end_zero;			     ///< pozostalosc z MLP2LN
    
    float *GetWeights();
    void ChangeSlope(float inc)
    {
        if(!freeze)
            slope+=inc;
    };

    void SetBackSignal(float bsignal)
    {
        backSignal = bsignal * diffActivation;
    }
    void SetSlope(float s)
    {
        if(!freeze) slope=s;
    };
    float GetSlope()
    {
        return slope;
    }

    void AddLayerToConnectionTable(Node *first_neuron_in_previus_layer);	//polaczenie wszystkich neuronow z poprzedzajacej warstwy z neuronem
    void AddNodeToConnectionTable(Node *neron_from_previus_layer);		//polaczenie neuronow
    void AddNodeToConnectionTable(Node *neron_from_previus_layer,float weight);  //to samo z inicjalizacja
    void DelLayerFromConnectionTable(Node *first_neuron_in_previus_layer);
    void DelNodeFromConnectionTable(Node *neron_from_previus_layer);
    void DelConnectionTable();			//usuniecie wszystkich polaczen z warstwa poprzednia

    void SetActivation(float value)
    {
        activation = value;
    };
    void SetWeight(Node *neron_from_previus_layer,float w);		//zmiana wagi dla danego polaczenia
    void SetWeights(float f);		// ustawia wszytkie wago neurony na dana wartosc
    void CopyWeightsFromNode(Node *n);
    void SetBias(float value){ bias = value;}
    int  Frozen();
    void FreezeNode();
    void UnFreezeNode();

//    void CalculateBackSignal(float output,float l_factor=0.0,float reward_factor=0.0,float penalty_factor=0.0);

    virtual void InitNode();				// losowa inicjalizacja wag
    virtual void CalculateBackSignal();		//backSignal = DivActiv * sum{w * BackSignal_polaczonego_neuronu}
    //virtual void ChangeWeights(float learning_factor,float regularization=0.0);	// zmiana wag zgodnie z gradientem
    virtual void ChangeWeights(const network_parameters &p)=0;
    
    virtual float GetRightBias(){return bias;}
    virtual float GetLeftBias(){return bias;}
    virtual void EstimateBias()=0;
    virtual void Activation()=0;			
    virtual void DiffActivation()=0;		//	diffActivation=slope*activation*(1-activation);
    virtual Node* CreateNode()=0;			//tworzenie neuronu takiego samego typu
    virtual Node *Clone()=0;				//to samo co wyzej ale kopiowane sa tez parametry neuronu
    
    virtual void CopyNode(Node *n);			//kopiowanie parametrow neuronu (bez kopiowania polaczen i wag)
    virtual void DisplayWeights();
    virtual ostream& DisplayWeights(ostream &o);
    virtual void SaveNeuronInformation(ofstream &);
    virtual void LoadNeuronInformation(ifstream &);

    virtual ~Node();
    
    float GetActivation()
    {
        return activation;
    };
    int  CheckIfFrozen();
    float GetProjection()
    {
        return projection;
    };
    float GetBias()
    {
        return bias;
    };

    void SaveConnection(ofstream &);

    int GetNumberOfNullConnections(float epsilon);
    
    friend ostream &operator<<(ostream &o,Node &n)
    {
        return n.DisplayWeights(o);
    }
};

class Sigmoid:public Node
{
    void Activation();
    void DiffActivation();

    float y_min,y_max; 
    
public:
    Sigmoid()
    {
        type=SIGMOID;
        name ="sigmoid";
        y_min=y_max = 0.0;
    };

    Node* CreateNode()
    {
        return new Sigmoid();
    }

    Node *Clone();
    void EstimateBias();
    void ChangeWeights(const network_parameters &p);
};

class LinearNode:public Node
{
    void Activation();
    void DiffActivation();
    float y_min,y_max; 
public:
    LinearNode()
    {
        name="Linear";
        type=LINEAR;
        y_min=y_max = 0.0;
    };

    Node* CreateNode()
    {
        return new LinearNode();
    };

    Node *Clone();
    void EstimateBias();
    void ChangeWeights(const network_parameters &p);
    ostream& DisplayWeights(ostream &o);
};

class Magic:public Node
{
    float activ1,activ2;
    float diffActiv1,diffActiv2;
    float backSignal1,backSignal2;
    float lc;           // 1- uczenie klastrow z 1, 0 - uczenie klastrow z 0
    float y_min,y_max;  //skrajne wartosci w*x - potrzebne do
    //wyznaczenia biasu

    void Activation();
    void DiffActivation();
    float bias2;
    
public:

    Magic();
    Magic(int learn_class);
    Node *CreateNode()
    {
        return new Magic();
    }

    Node *Clone();
    void EstimateBias();
//    void ChangeWeights(float learning_factor,float regularization=0.0);
    void ChangeWeights(const network_parameters &p);
    void CalculateBackSignal();
    void DisplayWeights();
    void InitNode();
    void InitBiases(Data *d);
    void SaveNeuronInformation(ofstream &);
    void LoadNeuronInformation(ifstream &);
    void CopyNode(Node *n);
    int GetWeights(float **tab);
    float GetLeftBias(){ return bias; }
    float GetRightBias() { return bias2;}
    
    ostream& DisplayWeights(ostream &o);
};


class BiCentral:public Node
{
    float activ1,activ2;
    float diffActiv1,diffActiv2;
    float backSignal1,backSignal2;
    float y_min,y_max;  //skrajne wartosci w*x - potrzebne do
    float expBias;
    
    void Activation();
    void DiffActivation();

public:

    float center;
    BiCentral();
    Node *CreateNode()
    {
        return new BiCentral();
    }

    Node *Clone();
    void EstimateBias();
//    void ChangeWeights(float learning_factor,float regularization=0.0);
    void ChangeWeights(const network_parameters &p);
    void CalculateBackSignal();
    void InitNode();
    void InitBiases(Data *d);
    void SaveNeuronInformation(ofstream &);
    void LoadNeuronInformation(ifstream &);
    void CopyNode(Node *n);
    int GetWeights(float **tab);
    float GetCenter( ) {return center;}
    float GetRightBias(){return (center + bias) ; }
    float GetLeftBias(){ return (center - bias) ; }
    float GetProjection()
    {
        return projection;
    }
    ostream& DisplayWeights(ostream &o);
    
    
    
 
    
};

#endif
