#ifndef __PARAMATERS_H
#define __PARAMATERS_H

#include <string.h>
#include<stdlib.h>
#include <iostream>
#include <fstream>
#include "ConfigFile.h"

using namespace std;
enum CV_type
{
	monte_carlo,
	stratified
};

enum strategy
{
    grow,
    static_network
};

enum neuron_type
{
    linear,
    sigmoid,
    bicentral,
    ssigmoid
};

enum initiation_type
{
    _rand,
    _copy
};

enum transformation
{
    none,
    normalization,
    standarization,
    binarization
};

enum data_format
{
    raw,         // simple matrix of float values with last column as  labels jest klasa
    snns,       // format snns
    sca          // format uzywany w artykule m. musseli Sequential Constructive Alg.
 };

enum network_type
{
    MLP,         // siec typowa mlp, problemy wieloklasowe
    bNet,   // siec z jedna wartowa ukryta i jednym zamrozonym wyjsciem
    kNet     // siec zlozona z K sieci binarnych, problemy wieloklasowe
 };
 
struct network_parameters
{
    float learning_rate;
    float reward_factor;
    float penalty_factor;
    float lambda_factor;
    float regularization;
    unsigned long max_nr_of_iterations;
    unsigned int initial_nr_of_hidden_neurons;
    unsigned int number_of_initiatializations;     // solution is obtained from best of N initiations
    int number_of_attempts;                        // how many repetitions of searching if no improvements was made
                                                   // where 0 = infinity
	 float eps;                                     // dokladnosc uczenia
	 
    strategy network_architecture;
    network_type networkType;
 	 int strategy_of_learning;
    float slope;
	 float force_zero;
	 float force_one;
	 float end_zero;
	 unsigned int average;
	 int slope_learning;
	 neuron_type input,hidden,output;
    initiation_type initiation;
    int hidden_neurons_count;
    bool vector_selection;
    bool hammingClustering;
    bool freezOutput;

	network_parameters():learning_rate(0.5),
						reward_factor(0.0),
						penalty_factor(0.0),
						lambda_factor(0.0),
						regularization(0.0),
						max_nr_of_iterations(1000),
						initial_nr_of_hidden_neurons(1),
						number_of_initiatializations(10),
						number_of_attempts(3),
						network_architecture(grow),
						strategy_of_learning(1),
						slope(2.0),
						force_zero(0.0),
						force_one(0.0),
						end_zero(0.01),
						slope_learning(0),
						average(50),
						input(linear),
						hidden(sigmoid),
						output(linear),
						initiation(_rand),
						hidden_neurons_count(1),
						vector_selection(false),
						hammingClustering(false),
						freezOutput(true),
						networkType(bNet),
						eps(0.1)				
										

	{};
};

/*
  struct learning_parameters
    {
        float learning_rate;      //wsp. uczenia
        float force_zero;       // wsp. kary pierwszy (wymuszanie zer i jedynek)
        float force_one;        // wsp. kary drugi - regularyzacja
        float end_zero;
        long max_iter;          // max. liczba iteracji
        int average;
        int init_slope;       // poczatkowe
        int strategy_of_learning;
        float reward_factor;
        float penalty_factor;
        float l_factor;
        int nr_of_initiations;
    } param;
*/
enum data_type
{
    table_with_labels,                 ///< simple table data with last col. as label
    list_of_ints,                       ///< each inteager value in file represents single boolean function
    generated                           ///< datafile is generated during process
};

struct data_parameters
{
//    char *input_file;	//dla badania funkcji boolowskich
	char *train_file;
	char *test_file;
   unsigned int dim;
   data_format d_format;
   float diffusion;
   transformation trans;
   data_type datatype;
//    char *datafile;
	data_parameters():diffusion(0.0f),dim(0),test_file(NULL),train_file(NULL),datatype(table_with_labels),trans(none),d_format(raw) {};
};

struct train_parameters
{
	unsigned int nr_cv;
	unsigned int nr_test;
	unsigned int cv_method;
	char *datasets_directory;
	train_parameters():nr_cv(10),nr_test(10),cv_method(1),datasets_directory(NULL)
	{};
};

class Parameters : public network_parameters, public data_parameters, public train_parameters
{
    char *config_file;
 public:
    int debug_level;

	 Parameters():config_file(NULL),debug_level(0){};
    void ReadConfigFile(char *file);
    void ParseArguments(int argc, char **argv);
    void DisplayParameters();
    void Usage();
};
#endif

